/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:	   projections.cpp         Projection data classes
**   Programmer:   Kevin Rosenberg
**   Date Started: Aug 84
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: projections.cpp,v 1.45 2001/01/16 20:55:29 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#include "ct.h"

const kuint16 Projections::m_signature = ('P'*256 + 'J');

const int Projections::POLAR_INTERP_INVALID = -1;
const int Projections::POLAR_INTERP_NEAREST = 0;
const int Projections::POLAR_INTERP_BILINEAR = 1;
const int Projections::POLAR_INTERP_BICUBIC = 2;

const char* Projections::s_aszInterpName[] = 
{
  {"nearest"},
  {"bilinear"},
//  {"bicubic"},
};

const char* Projections::s_aszInterpTitle[] = 
{
  {"Nearest"},
  {"Bilinear"},
//  {"Bicubic"},
};

const int Projections::s_iInterpCount = sizeof(s_aszInterpName) / sizeof(char*);


/* NAME
*    Projections		Constructor for projections matrix storage 
*
* SYNOPSIS
*    proj = projections_create (filename, nView, nDet)
*    Projections& proj		Allocated projections structure & matrix
*    int nView			Number of rotated view
*    int nDet			Number of detectors
*
*/

Projections::Projections (const Scanner& scanner)
: m_projData(0)
{
  initFromScanner (scanner);
}


Projections::Projections (const int nView, const int nDet)
: m_projData(0)
{
  init (nView, nDet);
}

Projections::Projections (void)
: m_projData(0)
{
  init (0, 0);
}

Projections::~Projections (void)
{
  deleteProjData();
}

int
Projections::convertInterpNameToID (const char* const interpName)
{
  int interpID = POLAR_INTERP_INVALID;
  
  for (int i = 0; i < s_iInterpCount; i++)
    if (strcasecmp (interpName, s_aszInterpName[i]) == 0) {
      interpID = i;
      break;
    }
    
    return (interpID);
}

const char*
Projections::convertInterpIDToName (const int interpID)
{
  static const char *interpName = "";
  
  if (interpID >= 0 && interpID < s_iInterpCount)
    return (s_aszInterpName[interpID]);
  
  return (interpName);
}

const char*
Projections::convertInterpIDToTitle (const int interpID)
{
  static const char *interpTitle = "";
  
  if (interpID >= 0 && interpID < s_iInterpCount)
    return (s_aszInterpTitle[interpID]);
  
  return (interpTitle);
}



void
Projections::init (const int nView, const int nDet)
{
  m_label.setLabelType (Array2dFileLabel::L_HISTORY);
  m_nView = nView;
  m_nDet = nDet;
  newProjData ();
  
  time_t t = time (NULL);
  tm* lt = localtime (&t);
  m_year = lt->tm_year;
  m_month = lt->tm_mon;
  m_day = lt->tm_mday;
  m_hour = lt->tm_hour;
  m_minute = lt->tm_min;
  m_second = lt->tm_sec;
}

void
Projections::initFromScanner (const Scanner& scanner)
{
  m_label.setLabelType (Array2dFileLabel::L_HISTORY);
  deleteProjData();
  init (scanner.nView(), scanner.nDet());
  
  m_phmLen = scanner.phmLen();
  m_rotInc = scanner.rotInc();
  m_detInc = scanner.detInc();
  m_geometry = scanner.geometry();
  m_focalLength = scanner.focalLength();
  m_fieldOfView = scanner.fieldOfView();
  m_rotStart = 0;
  m_detStart =  -(scanner.detLen() / 2);
}

void
Projections::setNView (int nView)  // used by MPI to reduce # of views
{
  deleteProjData();
  init (nView, m_nDet);
}

// NAME
// newProjData

void 
Projections::newProjData (void)
{
  if (m_projData)
    sys_error(ERR_WARNING, "m_projData != NULL [newProjData]");
  
  if (m_nView > 0 && m_nDet) {
    m_projData = new DetectorArray* [m_nView];
    
    for (int i = 0; i < m_nView; i++)
      m_projData[i] = new DetectorArray (m_nDet);
  }
}


/* NAME
*   projections_free			Free memory allocated to projections
*
* SYNOPSIS
*   projections_free(proj)
*   Projections& proj				Projectionss to be deallocated
*/

void 
Projections::deleteProjData (void)
{
  if (m_projData != NULL) {
    for (int i = 0; i < m_nView; i++)
      delete m_projData[i];
    
    delete m_projData;
    m_projData = NULL;
  }
}


/* NAME
*    Projections::headerWwrite         Write data header for projections file
*
*/

bool 
Projections::headerWrite (fnetorderstream& fs)
{
  kuint16 _hsize = m_headerSize;
  kuint16 _signature = m_signature;
  kuint32 _nView = m_nView;
  kuint32 _nDet = m_nDet;
  kuint32 _geom = m_geometry;
  kuint16 _remarksize = m_remark.length();
  kuint16 _year = m_year;
  kuint16 _month = m_month;
  kuint16 _day = m_day;
  kuint16 _hour = m_hour;
  kuint16 _minute = m_minute;
  kuint16 _second = m_second;
  
  kfloat64 _calcTime = m_calcTime;
  kfloat64 _rotStart = m_rotStart;
  kfloat64 _rotInc = m_rotInc;
  kfloat64 _detStart = m_detStart;
  kfloat64 _detInc = m_detInc;
  kfloat64 _phmLen = m_phmLen;
  kfloat64 _fieldOfView = m_fieldOfView;
  kfloat64 _focalLength = m_focalLength;
  
  fs.seekp(0);
  if (! fs)
    return false;
  
  fs.writeInt16 (_hsize);
  fs.writeInt16 (_signature);
  fs.writeInt32 (_nView);
  fs.writeInt32 (_nDet);
  fs.writeInt32 (_geom);
  fs.writeFloat64 (_calcTime);
  fs.writeFloat64 (_rotStart);
  fs.writeFloat64 (_rotInc);
  fs.writeFloat64 (_detStart);
  fs.writeFloat64 (_detInc);
  fs.writeFloat64 (_phmLen);
  fs.writeFloat64 (_focalLength);
  fs.writeFloat64 (_fieldOfView);
  fs.writeInt16 (_year);
  fs.writeInt16 (_month);
  fs.writeInt16 (_day);
  fs.writeInt16 (_hour);
  fs.writeInt16 (_minute);
  fs.writeInt16 (_second);
  fs.writeInt16 (_remarksize);
  fs.write (m_remark.c_str(), _remarksize);
  
  m_headerSize = fs.tellp();
  _hsize = m_headerSize;
  fs.seekp(0);
  fs.writeInt16 (_hsize);
  if (! fs)
    return false;
  
  return true;
}

/* NAME
*    projections_read_header         Read data header for projections file
*
*/
bool
Projections::headerRead (fnetorderstream& fs)
{
  kuint16 _hsize, _signature, _year, _month, _day, _hour, _minute, _second, _remarksize = 0;
  kuint32 _nView, _nDet, _geom;
  kfloat64 _calcTime, _rotStart, _rotInc, _detStart, _detInc, _phmLen, _focalLength, _fieldOfView;
  
  fs.seekg(0);
  if (! fs)
    return false;
  
  fs.readInt16 (_hsize);
  fs.readInt16 (_signature);
  fs.readInt32 (_nView);
  fs.readInt32 (_nDet);
  fs.readInt32 (_geom);
  fs.readFloat64 (_calcTime);
  fs.readFloat64 (_rotStart);
  fs.readFloat64 (_rotInc);
  fs.readFloat64 (_detStart);
  fs.readFloat64 (_detInc);
  fs.readFloat64 (_phmLen);
  fs.readFloat64 (_focalLength);
  fs.readFloat64 (_fieldOfView);
  fs.readInt16 (_year);
  fs.readInt16 (_month);
  fs.readInt16 (_day);
  fs.readInt16 (_hour);
  fs.readInt16 (_minute);
  fs.readInt16 (_second);
  fs.readInt16 (_remarksize);
  
  if (! fs) {
    sys_error (ERR_SEVERE, "Error reading header information , _remarksize=%d [projections_read_header]", _remarksize);
    return false;
  }
  
  if (_signature != m_signature) {
    sys_error (ERR_SEVERE, "File %s does not have a valid projection file signature", m_filename.c_str());
    return false;
  }
  
  char* pszRemarkStorage = new char [_remarksize+1];
  fs.read (pszRemarkStorage, _remarksize);
  if (! fs) {
    sys_error (ERR_SEVERE, "Error reading remark, _remarksize = %d", _remarksize);
    return false;
  }
  pszRemarkStorage[_remarksize] = 0;
  m_remark = pszRemarkStorage;
  delete pszRemarkStorage;
  
  off_t _hsizeread = fs.tellg();
  if (!fs || _hsizeread != _hsize) {
    sys_error (ERR_WARNING, "File header size read %ld != file header size stored %ld [read_projections_header]\n_remarksize=%ld", (long int) _hsizeread, _hsize, _remarksize);
    return false;
  }
  
  m_headerSize = _hsize;
  m_nView = _nView;
  m_nDet = _nDet;
  m_geometry = _geom;
  m_calcTime = _calcTime;
  m_rotStart = _rotStart;
  m_rotInc = _rotInc;
  m_detStart = _detStart;
  m_detInc = _detInc;
  m_phmLen = _phmLen;
  m_focalLength = _focalLength;
  m_fieldOfView = _fieldOfView;
  m_year = _year;
  m_month = _month;
  m_day = _day;
  m_hour = _hour;
  m_minute = _minute;
  m_second = _second;
  
  m_label.setLabelType (Array2dFileLabel::L_HISTORY);
  m_label.setLabelString (m_remark);
  m_label.setCalcTime (m_calcTime);
  m_label.setDateTime (m_year, m_month, m_day, m_hour, m_minute, m_second);
  
  return true;
}

bool
Projections::read (const std::string& filename)
{
  return read (filename.c_str());
}


bool
Projections::read (const char* filename)
{
  m_filename = filename;
#ifdef MSVC
  frnetorderstream fileRead (m_filename.c_str(), std::ios::in | std::ios::binary);
#else
  frnetorderstream fileRead (m_filename.c_str(), std::ios::in | std::ios::binary | std::ios::nocreate);
#endif
  
  if (fileRead.fail())
    return false;
  
  if (! headerRead (fileRead))
    return false;
  
  deleteProjData ();
  newProjData();
  
  for (int i = 0; i < m_nView; i++) {
    if (! detarrayRead (fileRead, *m_projData[i], i))
      break;
  }
  
  fileRead.close();
  return true;
}


bool 
Projections::copyViewData (const std::string& filename, std::ostream& os, int startView, int endView)
{
  return copyViewData (filename.c_str(), os, startView, endView);
}

bool 
Projections::copyViewData (const char* const filename, std::ostream& os, int startView, int endView)
{
  frnetorderstream is (filename, std::ios::in | std::ios::binary);
  kuint16 sizeHeader, signature;
  kuint32 _nView, _nDet;
  
  is.seekg (0);
  if (is.fail()) {
    sys_error (ERR_SEVERE, "Unable to read projection file %s", filename);
    return false;
  }

  is.readInt16 (sizeHeader);
  is.readInt16 (signature);
  is.readInt32 (_nView);
  is.readInt32 (_nDet);
  int nView = _nView;
  int nDet = _nDet;
  
  if (signature != m_signature) {
    sys_error (ERR_SEVERE, "Illegal signature in projection file %s", filename);
    return false;
  }
  
  if (startView < 0)
    startView = 0;
  if (startView > nView - 1)
    startView = nView;
  if (endView < 0 || endView > nView - 1)
    endView = nView - 1;
  
  if (startView > endView) { // swap if start > end
    int tempView = endView;
    endView = startView;
    startView = tempView;
  }
  
  int sizeView = 8 /* view_angle */ + 4 /* nDet */ + (4 * nDet);
  unsigned char* pViewData = new unsigned char [sizeView];
  
  for (int i = startView; i <= endView; i++) {
    is.seekg (sizeHeader + i * sizeView);
    is.read (reinterpret_cast<char*>(pViewData), sizeView);
    os.write (reinterpret_cast<char*>(pViewData), sizeView);
    if (is.fail() || os.fail())
      break;
  }
  
  delete pViewData;
  if (is.fail()) 
    sys_error (ERR_SEVERE, "Error reading projection file");
  if (os.fail()) 
    sys_error (ERR_SEVERE, "Error writing projection file");
  
  return (! (is.fail() | os.fail()));
}

bool 
Projections::copyHeader (const std::string& filename, std::ostream& os)
{
  return copyHeader (filename.c_str(), os);
}

bool
Projections::copyHeader (const char* const filename, std::ostream& os)
{
  frnetorderstream is (filename, std::ios::in | std::ios::binary);
  kuint16 sizeHeader, signature;
  is.readInt16 (sizeHeader);
  is.readInt16 (signature);
  is.seekg (0);
  if (signature != m_signature) {
    sys_error (ERR_SEVERE, "Illegal signature in projection file %s", filename);
    return false;
  }
  
  unsigned char* pHdrData = new unsigned char [sizeHeader];
  is.read (reinterpret_cast<char*>(pHdrData), sizeHeader);
  if (is.fail()) {
    sys_error (ERR_SEVERE, "Error reading header");
    return false;
  }
  
  os.write (reinterpret_cast<char*>(pHdrData), sizeHeader);
  if (os.fail()) {
    sys_error (ERR_SEVERE, "Error writing header");
    return false;
  }
  
  return true;
}

bool
Projections::write (const std::string& filename)
{
  return write (filename.c_str());
}

bool
Projections::write (const char* filename)
{
  frnetorderstream fs (filename, std::ios::out | std::ios::binary | std::ios::trunc | std::ios::ate);
  m_filename = filename;
  if (! fs) {
    sys_error (ERR_SEVERE, "Error opening file %s for output [projections_create]", filename);
    return false;
  }
  
  if (! headerWrite (fs))
    return false;
  
  if (m_projData != NULL) {
    for (int i = 0; i < m_nView; i++) {
      if (! detarrayWrite (fs, *m_projData[i], i))
        break;
    }
  }
  if (! fs)
    return false;
  
  fs.close();
  
  return true;
}

/* NAME
*   detarrayRead		Read a Detector Array structure from the disk
*
* SYNOPSIS
*   detarrayRead (proj, darray, view_num)
*   DETARRAY *darray		Detector array storage location to be filled
*   int      view_num		View number to read
*/

bool
Projections::detarrayRead (fnetorderstream& fs, DetectorArray& darray, const int iview)
{
  const int detval_bytes = darray.nDet() * sizeof(kfloat32);
  const int detheader_bytes = sizeof(kfloat64) /* view_angle */ + sizeof(kint32) /* nDet */;
  const int view_bytes = detheader_bytes + detval_bytes;
  const off_t start_data = m_headerSize + (iview * view_bytes);
  DetectorValue* detval_ptr = darray.detValues();  
  kfloat64 view_angle;
  kuint32 nDet;
  
  fs.seekg (start_data);
  
  fs.readFloat64 (view_angle);
  fs.readInt32 (nDet);
  darray.setViewAngle (view_angle);
  //  darray.setNDet ( nDet);
  
  for (unsigned int i = 0; i < nDet; i++) {
    kfloat32 detval;
    fs.readFloat32 (detval);
    detval_ptr[i] = detval;
  }
  if (! fs)
    return false;
  
  return true;
}


/* NAME
*   detarrayWrite			Write detector array data to the disk
*
* SYNOPSIS
*   detarrayWrite (darray, view_num)
*   DETARRAY *darray			Detector array data to be written
*   int      view_num			View number to write
*
* DESCRIPTION
*       This routine writes the detarray data from the disk sequentially to
*    the file that was opened with open_projections().  Data is written in
*    binary format.
*/

bool
Projections::detarrayWrite (fnetorderstream& fs, const DetectorArray& darray, const int iview)
{
  const int detval_bytes = darray.nDet() * sizeof(float);
  const int detheader_bytes = sizeof(kfloat64) /* view_angle */ + sizeof(kint32) /* nDet */;
  const int view_bytes = detheader_bytes + detval_bytes;
  const off_t start_data = m_headerSize + (iview * view_bytes);
  const DetectorValue* const detval_ptr = darray.detValues();  
  kfloat64 view_angle = darray.viewAngle();
  kuint32 nDet = darray.nDet();
  
  fs.seekp (start_data);
  if (! fs) {
    sys_error (ERR_SEVERE, "Error seeking detectory array [detarrayWrite]");
    return false;
  }
  
  fs.writeFloat64 (view_angle);
  fs.writeInt32 (nDet);
  
  for (unsigned int i = 0; i < nDet; i++) {
    kfloat32 detval = detval_ptr[i];
    fs.writeFloat32 (detval);
  }
  
  if (! fs)
    return (false);
  
  return true;
}

/* NAME
*   printProjectionData			Print projections data
*
* SYNOPSIS
*   printProjectionData ()
*/

void
Projections::printProjectionData ()
{
  printProjectionData (0, nView() - 1);
}

void
Projections::printProjectionData (int startView, int endView)
{
  printf("Projections Data\n\n");
  printf("Description: %s\n", m_remark.c_str());
  printf("Geometry: %s\n", Scanner::convertGeometryIDToName (m_geometry));
  printf("nView       = %8d           nDet = %8d\n", m_nView, m_nDet);
  printf("focalLength = %8.4f  fieldOfView = %8.4f\n", m_focalLength, m_fieldOfView);
  printf("rotStart    = %8.4f       rotInc = %8.4f\n", m_rotStart, m_rotInc);
  printf("detStart    = %8.4f       detInc = %8.4f\n", m_detStart, m_detInc);
  if (m_projData != NULL) {
    if (startView < 0)
      startView = 0;
    if (endView < 0)
      endView = m_nView - 1;
    if (startView > m_nView - 1)
      startView = m_nView - 1;
    if (endView > m_nView - 1)
      endView = m_nView - 1;
    for (int ir = startView; ir <= endView - 1; ir++) {
      printf("View %d: angle %f\n", ir, m_projData[ir]->viewAngle());
      DetectorValue* detval = m_projData[ir]->detValues();
      for (int id = 0; id < m_projData[ir]->nDet(); id++)
        printf("%8.4f  ", detval[id]);
      printf("\n");
    }
  }
}

void 
Projections::printScanInfo (std::ostringstream& os) const
{
  os << "Number of detectors: " << m_nDet << "\n";
  os << "    Number of views: " << m_nView<< "\n";
  os << "             Remark: " << m_remark.c_str()<< "\n";
  os << "           Geometry: " << Scanner::convertGeometryIDToName (m_geometry)<< "\n";
  os << "       Focal Length: " << m_focalLength<< "\n";
  os << "      Field Of View: " << m_fieldOfView<< "\n";
  os << "             phmLen: " << m_phmLen<< "\n";
  os << "           detStart: " << m_detStart<< "\n";
  os << "             detInc: " << m_detInc<< "\n";
  os << "           rotStart: " << m_rotStart<< "\n";
  os << "             rotInc: " << m_rotInc<< "\n";
}


bool 
Projections::convertPolar (ImageFile& rIF, int iInterpolationID)
{
  unsigned int nx = rIF.nx();
  unsigned int ny = rIF.ny();
  ImageFileArray v = rIF.getArray();
  ImageFileArray vImag = rIF.getImaginaryArray();

  if (! v || nx == 0 || ny == 0)
    return false;
  
  Array2d<double> adView (nx, ny);
  Array2d<double> adDet (nx, ny);
  double** ppdView = adView.getArray();
  double** ppdDet = adDet.getArray();

  calcArrayPolarCoordinates (nx, ny, ppdView, ppdDet);

  std::complex<double>** ppcDetValue = new std::complex<double>* [m_nView];
  unsigned int iView;
  for (iView = 0; iView < m_nView; iView++) {
    ppcDetValue[iView] = new std::complex<double> [m_nDet];
    for (unsigned int iDet = 0; iDet < m_nDet; iDet++)
      ppcDetValue[iView][iDet] = std::complex<double>(getDetectorArray (iView).detValues()[iDet], 0);
  }

  interpolatePolar (v, vImag, nx, ny, ppcDetValue, ppdView, ppdDet, m_nView, m_nDet, iInterpolationID);

  for (iView = 0; iView < m_nView; iView++)
    delete [] ppcDetValue[iView];
  delete [] ppcDetValue;

  return true;
}


bool 
Projections::convertFFTPolar (ImageFile& rIF, int iInterpolationID, int iZeropad)
{
  unsigned int nx = rIF.nx();
  unsigned int ny = rIF.ny();
  ImageFileArray v = rIF.getArray();
  if (! rIF.isComplex())
    rIF.convertRealToComplex();
  ImageFileArray vImag = rIF.getImaginaryArray();

  if (! v || nx == 0 || ny == 0)
    return false;
  
#ifndef HAVE_FFT
  return false;
#else
  Array2d<double> adView (nx, ny);
  Array2d<double> adDet (nx, ny);
  double** ppdView = adView.getArray();
  double** ppdDet = adDet.getArray();

  std::complex<double>** ppcDetValue = new std::complex<double>* [m_nView];
  unsigned int iView;
  double* pdDet = new double [m_nDet];
  fftw_complex* pcIn = new fftw_complex [m_nDet];
  fftw_plan plan = fftw_create_plan (m_nDet, FFTW_FORWARD, FFTW_IN_PLACE);

  for (iView = 0; iView < m_nView; iView++) {
    unsigned int iDet;
    for (iDet = 0; iDet < m_nDet; iDet++) {
      pcIn[iDet].re = getDetectorArray(iView).detValues()[iDet];
      pcIn[iDet].im = 0;
    }
    fftw_one (plan, pcIn, NULL);
    ppcDetValue[iView] = new std::complex<double> [m_nDet];
    for (iDet = 0; iDet < m_nDet; iDet++)
      ppcDetValue[iView][iDet] = std::complex<double> (pcIn[iDet].re, pcIn[iDet].im); 
    Fourier::shuffleFourierToNaturalOrder (ppcDetValue[iView], m_nDet);
  }

  fftw_destroy_plan (plan);  
  delete [] pcIn;
  
  calcArrayPolarCoordinates (nx, ny, ppdView, ppdDet);

  interpolatePolar (v, vImag, nx, ny, ppcDetValue, ppdView, ppdDet, m_nView, m_nDet, iInterpolationID);

  for (iView = 0; iView < m_nView; iView++)
    delete [] ppcDetValue[iView];
  delete [] ppcDetValue;

  return true;
#endif
}


void
Projections::calcArrayPolarCoordinates (unsigned int nx, unsigned int ny, double** ppdView, double** ppdDet)
{
  double xMin = -m_phmLen / 2;
  double xMax = xMin + m_phmLen;
  double yMin = -m_phmLen / 2;
  double yMax = yMin + m_phmLen;
  
  double xInc = (xMax - xMin) / nx;	// size of cells
  double yInc = (yMax - yMin) / ny;
  
  int iDetCenter = (m_nDet - 1) / 2;	// index refering to L=0 projection 

  if (m_geometry != Scanner::GEOMETRY_PARALLEL) {
    sys_error (ERR_WARNING, "convertPolar supports Parallel only");
    return;
  }
  
  // Calculates polar coordinates (view#, det#) for each point on phantom grid
  double x = xMin + xInc / 2;	// Rectang coords of center of pixel 
  for (unsigned int ix = 0; ix < nx; x += xInc, ix++) {
    double y = yMin + yInc / 2;
    for (unsigned int iy = 0; iy < ny; y += yInc, iy++) {
      double r = ::sqrt (x * x + y * y);
      double phi = atan2 (y, x);

      if (phi >= PI) {
        phi -= PI;
      } else if (phi < 0) {
        phi += PI;
      } else
        r = -r;
      
      ppdView[ix][iy] = (phi - m_rotStart) / m_rotInc;
      ppdDet[ix][iy] = (r / m_detInc) + iDetCenter;
    }
  }
}

void
Projections::interpolatePolar (ImageFileArray& v, ImageFileArray& vImag,
     unsigned int nx, unsigned int ny, std::complex<double>** ppcDetValue,
     double** ppdView, double** ppdDet, unsigned int nView, unsigned int nDet, int iInterpolationID)
{
  for (unsigned int ix = 0; ix < ny; ix++) {
    for (unsigned int iy = 0; iy < ny; iy++) {
      if (iInterpolationID == POLAR_INTERP_NEAREST) {
        int iView = nearest<int> (ppdView[ix][iy]);
        int iDet = nearest<int> (ppdDet[ix][iy]);
        if (iView == nView) {
          iView = 0;
       //   iDet = m_nDet - iDet;
        }
        if (iDet >= 0 && iDet < nDet && iView >= 0 && iView < nView) {
          v[ix][iy] = ppcDetValue[iView][iDet].real();
          if (vImag)
            vImag[ix][iy] = ppcDetValue[iView][iDet].imag();
        } else {
          sys_error (ERR_SEVERE, "Can't find projection data for ix=%d,iy=%d with radView=%f and radDet=%f", 
            ix, iy, ppdView[ix][iy], ppdDet[ix][iy]);
          v[ix][iy] = 0;
        }
      } else if (iInterpolationID == POLAR_INTERP_BILINEAR) {
        int iFloorView = static_cast<int>(ppdView[ix][iy]);
        double dFracView = ppdView[ix][iy] - iFloorView;
        int iFloorDet = static_cast<int>(ppdDet[ix][iy]);
        double dFracDet = ppdDet[ix][iy] - iFloorDet;

        if (iFloorDet >= 0 && iFloorView >= 0) { 
          std::complex<double> v1 = ppcDetValue[iFloorView][iFloorDet];
          std::complex<double> v2, v3, v4;
          if (iFloorView < nView - 1)
            v2 = ppcDetValue[iFloorView + 1][iFloorDet];
          else 
            v2 = ppcDetValue[0][iFloorDet];
          if (iFloorDet < nDet - 1) 
            v4 = ppcDetValue[iFloorView][iFloorDet+1];
          else
            v4 = v1;
          if (iFloorView < nView - 1 && iFloorDet < nDet - 1)
            v3 = ppcDetValue [iFloorView+1][iFloorDet+1];
          else if (iFloorView < nView - 1)
            v3 = v2;
          else 
            v3 = ppcDetValue[0][iFloorDet+1];
          std::complex<double> vInterp = (1 - dFracView) * (1 - dFracDet) * v1 +
            dFracView * (1 - dFracDet) * v2 + dFracView * dFracDet * v3 +
            dFracDet * (1 - dFracView) * v4;
          v[ix][iy] = vInterp.real();
          if (vImag)
            vImag[ix][iy] = vInterp.imag();
        } else {
          sys_error (ERR_SEVERE, "Can't find projection data for ix=%d,iy=%d with radView=%f and radDet=%f", 
            ix, iy, ppdView[ix][iy], ppdDet[ix][iy]);
          v[ix][iy] = 0;
          if (vImag)
            vImag[ix][iy] = 0;
        }
      } else if (iInterpolationID == POLAR_INTERP_BICUBIC) {
        v[ix][iy] =0;
          if (vImag)
            vImag[ix][iy] = 0;
      }
    }
  }
}


