/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:          ctsim.cpp
**   Purpose:       Top-level routines of CTSim program
**   Programmer:    Kevin Rosenberg
**   Date Started:  July 2000
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: ctsim.cpp,v 1.39 2001/01/20 08:10:33 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#ifdef MSVC
#define strdup _strdup
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/image.h"
#include "wx/filesys.h"
#include "wx/fs_zip.h"

#if !wxUSE_DOC_VIEW_ARCHITECTURE
#error You must set wxUSE_DOC_VIEW_ARCHITECTURE to 1 in setup.h!
#endif


#include "ct.h"
#include "ctsim.h"
#include "ctsim-map.h"
#include "docs.h"
#include "views.h"
#include "dialogs.h"

#if defined(HAVE_CONFIG_H)
#include "config.h"
#endif

#if defined(HAVE_GETOPT_H) || defined(HAVE_GETOPT_LONG)
#ifdef MSVC
#define __STDC__ 1
#endif
#include "getopt.h"
#ifdef MSVC
#undef __STDC__
#endif
#endif

static const char* rcsindent = "$Id: ctsim.cpp,v 1.39 2001/01/20 08:10:33 kevin Exp $";

struct option CTSimApp::ctsimOptions[] = 
{
  {"help", 0, 0, O_HELP},
  {"version", 0, 0, O_VERSION},
  {0, 0, 0, 0}
};

IMPLEMENT_APP(CTSimApp)

CTSimApp::CTSimApp()
: m_docManager(NULL), m_pFrame(NULL)
{
  theApp = this;
}

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif

bool
CTSimApp::OnInit()
{
#ifdef HAVE_SETPRIORITY
  setpriority (PRIO_PROCESS, 0, 15);  // set to low scheduling priority
#endif
  
  g_bRunningWXWindows = true;
  // process options
  while (1) {
    int c = getopt_long (argc, argv, "", ctsimOptions, NULL);
    if (c == -1)
      break;
    
    switch (c) {
    case O_VERSION:
      std::cout << rcsindent << std::endl;
#ifdef CTSIMVERSION
      std::cout << "Version: CTSIMVERSION" << std::endl;
#elif defined(VERSION)
      std::cout << "Version: VERSION" << std::endl;
#endif
      exit(0);
    case O_HELP:
    case '?':
      usage (argv[0]);
      exit (0);
    default:
      usage (argv[0]);
      exit (1);
    }
  }
  
  m_docManager = new wxDocManager (wxDEFAULT_DOCMAN_FLAGS, true);
  
  new wxDocTemplate (m_docManager, "ImageFile", "*.if", "", "if", "ImageFile doc", "ImageFile View", CLASSINFO(ImageFileDocument), CLASSINFO(ImageFileView));
  new wxDocTemplate (m_docManager, "ProjectionFile", "*.pj", "", "pj", "ProjectionFile doc", "ProjectionFile View", CLASSINFO(ProjectionFileDocument), CLASSINFO(ProjectionFileView));
  new wxDocTemplate (m_docManager, "PhantomFile", "*.phm", "", "phm", "Phantom doc", "Phantom View", CLASSINFO(PhantomDocument), CLASSINFO(PhantomView));
  new wxDocTemplate (m_docManager, "PlotFile", "*.plt", "", "plt", "Plot doc", "Plot View", CLASSINFO(PlotFileDocument), CLASSINFO(PlotFileView));
  
#if wxUSE_GIF
  wxImage::AddHandler(new wxGIFHandler);     // Required for images in the online documentation
#endif
  
#if wxUSE_STREAMS && wxUSE_ZIPSTREAM && wxUSE_ZLIB
  wxFileSystem::AddHandler(new wxZipFSHandler);     // Required for advanced HTML help
#endif
  
  // Create the main frame window
  m_pFrame = new MainFrame(m_docManager, (wxFrame *) NULL, -1, "CTSim", wxPoint(0, 0), wxSize(500, 400), wxDEFAULT_FRAME_STYLE);
  
  SetTopWindow (m_pFrame);
  m_pFrame->Centre(wxBOTH);
  
  m_pFrame->Show(true);
  
  for (int i = optind + 1; i <= argc; i++) {
    wxString filename = argv [i - 1];
    m_docManager->CreateDocument (filename, wxDOC_SILENT);
  }
  
  setIconForFrame (m_pFrame);
  
#ifdef CTSIM_WINHELP
  if (! m_pFrame->getWinHelpController().Initialize("ctsim"))
    ::wxMessageBox ("Cannot initialize the help system", "Error");
#endif
  if (! m_pFrame->getHtmlHelpController().Initialize(::wxGetCwd() + "/ctsim"))
    ::wxMessageBox ("Cannot initialize the help system", "Error");
  
  return true;
}


#include "./ctsim.xpm"
void
CTSimApp::setIconForFrame(wxFrame* pFrame)
{
  wxIcon iconApp (ctsim16_xpm);
  
  if (iconApp.Ok())
    pFrame->SetIcon (iconApp);
}

void
CTSimApp::usage(const char* program)
{
  std::cout << "usage: " << fileBasename(program) << " [files-to-open...] [OPTIONS]\n";
  std::cout << "Computed Tomography Simulator (Graphical Shell)\n";
  std::cout << "\n";
  std::cout << "  --version Display version\n";
  std::cout << "  --help    Display this help message\n";
}

int
CTSimApp::OnExit()
{
  delete m_docManager;
#ifdef HAVE_DMALLOC
  dmalloc_shutdown();
#endif
  return 0;
}

wxString
CTSimApp::getUntitledFilename()
{
  static int untitledNumber = 1;
  
  wxString filename ("Untitled");
  filename << untitledNumber++;
  
  return (filename);
}


// Top-level window for CTSim

#if CTSIM_MDI
IMPLEMENT_CLASS(MainFrame, wxMDIParentFrame)

BEGIN_EVENT_TABLE(MainFrame, wxMDIParentFrame)
#else
IMPLEMENT_CLASS(MainFrame, wxDocParentFrame)

BEGIN_EVENT_TABLE(MainFrame, wxDocParentFrame)
#endif

EVT_MENU(MAINMENU_HELP_ABOUT, MainFrame::OnAbout)
EVT_MENU(MAINMENU_HELP_CONTENTS, MainFrame::OnHelpContents)
EVT_MENU(MAINMENU_HELP_TOPICS, MainFrame::OnHelpTopics)
EVT_SIZE(MainFrame::OnSize)

#if defined(CTSIM_WINHELP) && (defined(DEBUG) || defined(_DEBUG))
EVT_MENU(MAINMENU_HELP_SECONDARY, MainFrame::OnHelpSecondary)
#endif
EVT_MENU(MAINMENU_FILE_CREATE_PHANTOM, MainFrame::OnCreatePhantom)
EVT_MENU(MAINMENU_FILE_CREATE_FILTER, MainFrame::OnCreateFilter)
EVT_MENU(MAINMENU_FILE_EXIT, MainFrame::OnExit)
EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, MainFrame::OnMRUFile)
EVT_MENU(MAINMENU_WINDOW_BASE, MainFrame::OnWindowMenu0)
EVT_MENU(MAINMENU_WINDOW_BASE+1, MainFrame::OnWindowMenu1)
EVT_MENU(MAINMENU_WINDOW_BASE+2, MainFrame::OnWindowMenu2)
EVT_MENU(MAINMENU_WINDOW_BASE+3, MainFrame::OnWindowMenu3)
EVT_MENU(MAINMENU_WINDOW_BASE+4, MainFrame::OnWindowMenu4)
EVT_MENU(MAINMENU_WINDOW_BASE+5, MainFrame::OnWindowMenu5)
EVT_MENU(MAINMENU_WINDOW_BASE+6, MainFrame::OnWindowMenu6)
EVT_MENU(MAINMENU_WINDOW_BASE+7, MainFrame::OnWindowMenu7)
EVT_MENU(MAINMENU_WINDOW_BASE+8, MainFrame::OnWindowMenu8)
EVT_MENU(MAINMENU_WINDOW_BASE+9, MainFrame::OnWindowMenu9)
EVT_MENU(MAINMENU_WINDOW_BASE+10, MainFrame::OnWindowMenu10)
EVT_MENU(MAINMENU_WINDOW_BASE+11, MainFrame::OnWindowMenu11)
EVT_MENU(MAINMENU_WINDOW_BASE+12, MainFrame::OnWindowMenu12)
EVT_MENU(MAINMENU_WINDOW_BASE+13, MainFrame::OnWindowMenu13)
EVT_MENU(MAINMENU_WINDOW_BASE+14, MainFrame::OnWindowMenu14)
EVT_MENU(MAINMENU_WINDOW_BASE+15, MainFrame::OnWindowMenu15)
EVT_MENU(MAINMENU_WINDOW_BASE+16, MainFrame::OnWindowMenu16)
EVT_MENU(MAINMENU_WINDOW_BASE+17, MainFrame::OnWindowMenu17)
EVT_MENU(MAINMENU_WINDOW_BASE+18, MainFrame::OnWindowMenu18)
EVT_MENU(MAINMENU_WINDOW_BASE+19, MainFrame::OnWindowMenu19)
EVT_UPDATE_UI_RANGE(MAINMENU_WINDOW_BASE, MAINMENU_WINDOW_BASE+20, MainFrame::OnUpdateUI)
END_EVENT_TABLE()



#if CTSIM_MDI
MainFrame::MainFrame(wxDocManager *manager, wxFrame *frame, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, const long type)
: wxMDIParentFrame(NULL,  id, title, pos, size, type), m_pLog(NULL)
#else
MainFrame::MainFrame(wxDocManager *manager, wxFrame *frame, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, const long type)
: wxDocParentFrame(manager, frame, id, title, pos, size, type), m_pLog(NULL)
#endif
{
#if 0
//#if CTSIM_MDI
  m_pLog = new wxLogWindow (this, "Log");
  //std::cout = new ostream (m_pLog->GetTextCtrl());
  wxLog::SetActiveTarget(m_pLog);
#else
  m_pLog = new wxTextCtrl (this, -1, "Log Window\n", wxPoint(0, 0), wxSize(0,0), wxTE_MULTILINE | wxTE_READONLY);
  wxLog::SetActiveTarget (new wxLogTextCtrl(m_pLog));
#endif

  //// Make a menubar
  wxMenu *file_menu = new wxMenu;
  
  file_menu->Append(MAINMENU_FILE_CREATE_PHANTOM, "Cr&eate Phantom...\tCtrl-P");
  file_menu->Append(MAINMENU_FILE_CREATE_FILTER, "Create &Filter...\tCtrl-F");
  file_menu->Append(wxID_OPEN, "&Open...\tCtrl-O");
  
  file_menu->AppendSeparator();
  file_menu->Append(MAINMENU_FILE_EXIT, "E&xit");
  
  //  history of files visited
  theApp->getDocManager()->FileHistoryUseMenu(file_menu);

#ifndef CTSIM_MDI
  m_pWindowMenu = new wxMenu;
  m_pWindowMenu->UpdateUI (this);
#endif

  wxMenu* help_menu = new wxMenu;
  help_menu->Append(MAINMENU_HELP_CONTENTS, "&Contents\tF1");
  help_menu->Append(MAINMENU_HELP_TOPICS, "&Topics\tCtrl-T");
#if defined(CTSIM_WINHELP) && (defined(DEBUG) || defined(_DEBUG))
  help_menu->Append(MAINMENU_HELP_SECONDARY, "&Secondary Help");
#endif
  help_menu->Append(MAINMENU_HELP_ABOUT, "&About");
  
  wxMenuBar* menu_bar = new wxMenuBar;
  
  menu_bar->Append(file_menu, "&File");
#ifndef CTSIM_MDI
  menu_bar->Append(m_pWindowMenu, "&Window");
#endif
  menu_bar->Append(help_menu, "&Help");
  
  SetMenuBar(menu_bar);
  
#ifndef CTSIM_MDI
  for (int i = 0; i < MAX_WINDOW_MENUITEMS; i++) {
    m_apWindowMenuItems[i] = new wxMenuItem (m_pWindowMenu, MAINMENU_WINDOW_BASE+i, wxString("<Empty>"));
    m_pWindowMenu->Append (m_apWindowMenuItems[i]);
    m_pWindowMenu->Enable (MAINMENU_WINDOW_BASE+i, false);
  }
#endif

  m_iDefaultPhantomID = Phantom::PHM_HERMAN;
  m_iDefaultFilterID = SignalFilter::FILTER_BANDLIMIT;
  m_iDefaultFilterDomainID = SignalFilter::DOMAIN_FREQUENCY;
  m_iDefaultFilterXSize = 256;
  m_iDefaultFilterYSize = 256;
  m_dDefaultFilterParam = 1.;
  m_dDefaultFilterBandwidth = 1.;
  m_dDefaultFilterInputScale = 1.;
  m_dDefaultFilterOutputScale = 1.;
  
  wxAcceleratorEntry accelEntries[15];
  accelEntries[0].Set (wxACCEL_CTRL, static_cast<int>('O'), wxID_OPEN);
  accelEntries[1].Set (wxACCEL_CTRL, static_cast<int>('H'), MAINMENU_HELP_TOPICS);
  accelEntries[2].Set (wxACCEL_CTRL, static_cast<int>('P'), MAINMENU_FILE_CREATE_PHANTOM);
  accelEntries[3].Set (wxACCEL_CTRL, static_cast<int>('F'), MAINMENU_FILE_CREATE_FILTER);
#ifndef CTSIM_MDI
  accelEntries[4].Set (wxACCEL_NORMAL, WXK_F1, MAINMENU_HELP_CONTENTS);
  for (i = 0; i < 10; i++)
    accelEntries[i+4].Set (wxACCEL_CTRL, static_cast<int>('0'+i), MAINMENU_WINDOW_BASE+i);
  wxAcceleratorTable accelTable (15, accelEntries);
#else
  wxAcceleratorTable accelTable (4, accelEntries);
#endif

  SetAcceleratorTable (accelTable);
}

void
MainFrame::OnSize (wxSizeEvent& event)
{
  wxSize sizeClient = GetClientSize();
  wxSize sizeLog = sizeClient;
  if (sizeClient.y < 100) {
    sizeLog.y = 0;
    sizeLog.x = 0;
  } else {
    sizeLog.x = sizeClient.x;
    sizeLog.y = 100;
  }
  sizeClient.x -= sizeLog.x;
  sizeClient.y -= sizeLog.y;
//  m_pLog->SetSize (0, sizeClient.y, sizeLog.x, sizeLog.y);
//  GetClientWindow()->SetSize (0, 0, sizeClient.x, sizeClient.y);
//    GetClientWindow()->Refresh();
  wxMDIParentFrame::OnSize (event);
}

void 
MainFrame::OnCreatePhantom(wxCommandEvent& event)
{
  DialogGetPhantom dialogPhantom (this, m_iDefaultPhantomID);
  int dialogReturn = dialogPhantom.ShowModal();
  if (dialogReturn == wxID_OK) {
    wxString selection (dialogPhantom.getPhantom());
    *theApp->getLog() << "Selected phantom " << selection.c_str() << "\n";
    wxString filename = selection + ".phm";
    m_iDefaultPhantomID = Phantom::convertNameToPhantomID (selection.c_str());
    theApp->getDocManager()->CreateDocument(filename, wxDOC_SILENT);
  }
  
}

void 
MainFrame::OnCreateFilter (wxCommandEvent& WXUNUSED(event))
{
  DialogGetFilterParameters dialogFilter (this, m_iDefaultFilterXSize, m_iDefaultFilterYSize, m_iDefaultFilterID, m_dDefaultFilterParam, m_dDefaultFilterBandwidth, m_iDefaultFilterDomainID, m_dDefaultFilterInputScale, m_dDefaultFilterOutputScale);
  int dialogReturn = dialogFilter.ShowModal();
  if (dialogReturn == wxID_OK) {
    wxString strFilter (dialogFilter.getFilterName());
    wxString strDomain (dialogFilter.getDomainName());
    m_iDefaultFilterID = SignalFilter::convertFilterNameToID (strFilter.c_str());
    m_iDefaultFilterDomainID = SignalFilter::convertDomainNameToID (strDomain.c_str());
    m_iDefaultFilterXSize = dialogFilter.getXSize();
    m_iDefaultFilterYSize = dialogFilter.getYSize();
    m_dDefaultFilterBandwidth = dialogFilter.getBandwidth();
    m_dDefaultFilterParam= dialogFilter.getFilterParam();
    m_dDefaultFilterInputScale = dialogFilter.getInputScale();
    m_dDefaultFilterOutputScale = dialogFilter.getOutputScale();
    std::ostringstream os;
    os << "Generate Filter=" << strFilter.c_str() 
      << ", size=(" << static_cast<int>(m_iDefaultFilterXSize) << "," << static_cast<int>(m_iDefaultFilterYSize) 
      << "), domain=" << strDomain.c_str() << ", filterParam=" << m_dDefaultFilterParam << ", bandwidth=" << m_dDefaultFilterBandwidth 
      << ", inputScale=" << m_dDefaultFilterInputScale << ", outputScale=" << m_dDefaultFilterOutputScale;
    *theApp->getLog() << os.str().c_str() << "\n";
    wxString filename = "untitled.if";
    ImageFileDocument* pFilterDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument ("untitled.if", wxDOC_SILENT));
    if (! pFilterDoc) {
      sys_error (ERR_SEVERE, "Unable to create filter image");
      return;
    }
    ImageFile& rIF = pFilterDoc->getImageFile();
    rIF.setArraySize (m_iDefaultFilterXSize, m_iDefaultFilterYSize);
    rIF.filterResponse (strDomain.c_str(), m_dDefaultFilterBandwidth, strFilter.c_str(), m_dDefaultFilterParam, m_dDefaultFilterInputScale, m_dDefaultFilterOutputScale);
    rIF.labelAdd (os.str().c_str());
    if (theApp->getSetModifyNewDocs())
      pFilterDoc->Modify (true);
    pFilterDoc->UpdateAllViews();
    pFilterDoc->GetFirstView()->OnUpdate (NULL, NULL);
  }
}

void
CTSimApp::getCompatibleImages (const ImageFileDocument* pIFDoc, std::vector<ImageFileDocument*>& vecIF)
{
  const ImageFile& rIF = pIFDoc->getImageFile();
  unsigned int nx = rIF.nx();
  unsigned int ny = rIF.ny();
  wxList& rListDocs = m_docManager->GetDocuments();
  for (wxNode* pNode = rListDocs.GetFirst(); pNode != NULL; pNode = pNode->GetNext()) {
    wxDocument* pDoc = reinterpret_cast<wxDocument*>(pNode->GetData());
    ImageFileDocument* pIFCompareDoc = dynamic_cast<ImageFileDocument*>(pDoc);
    if (pIFCompareDoc && (pIFDoc != pIFCompareDoc)) {
      const ImageFile& rCompareIF = pIFCompareDoc->getImageFile();
      if (rCompareIF.nx() == nx && rCompareIF.ny() == ny)
        vecIF.push_back (pIFCompareDoc);
    }
  }
}

void 
MainFrame::OnHelpTopics (wxCommandEvent& event)
{
  showHelp (event.GetId());
}

void 
MainFrame::OnHelpContents (wxCommandEvent& event)
{
  showHelp (event.GetId());
}

#if defined(CTSIM_WINHELP) && (defined(DEBUG) || defined(_DEBUG))
void
MainFrame::OnHelpSecondary (wxCommandEvent& event)
{
    m_htmlHelp.DisplayContents();
}
#endif

void
MainFrame::showHelp (int commandID)
{
#ifdef CTSIM_WINHELP
  m_winHelp.LoadFile();
#else
  m_htmlHelp.LoadFile();
#endif
  
  switch (commandID) {

  case MAINMENU_HELP_CONTENTS:
#ifdef CTSIM_WINHELP
    m_winHelp.DisplayContents ();
#else
    m_htmlHelp.DisplayContents ();
#endif
    break;

  case MAINMENU_HELP_TOPICS:
#ifdef CTSIM_WINHELP
    m_winHelp.DisplaySection (introduction);
#else
    m_htmlHelp.DisplayIndex();
#endif
    break;

  default:
    *getLog() << "Unknown help command # " << commandID << "\n";
    break;
  }
}

void 
MainFrame::OnExit (wxCommandEvent& WXUNUSED(event) )
{
  Close(true);
}

void
MainFrame::OnUpdateUI (wxUpdateUIEvent& rEvent)
{
  int iPos = 0;
  wxList& rListDocs = theApp->getDocManager()->GetDocuments();
  wxNode* pNode = rListDocs.GetFirst();
  while (iPos < MAX_WINDOW_MENUITEMS && pNode != NULL) {
    wxDocument* pDoc = static_cast<wxDocument*>(pNode->GetData());
    wxString strFilename = pDoc->GetFilename();
    if (iPos < 10) {
      strFilename += "\tCtrl-";
      strFilename += static_cast<char>('0' + iPos);
    }
    static_cast<wxMenuItemBase*>(m_apWindowMenuItems[iPos])->SetName (strFilename);
    m_apWindowMenuData[iPos] = pDoc;
    m_pWindowMenu->Enable (MAINMENU_WINDOW_BASE+iPos, true);
    iPos++;
    pNode = pNode->GetNext();
  }
  for (int i = iPos; i < MAX_WINDOW_MENUITEMS; i++) {
    m_pWindowMenu->Enable (MAINMENU_WINDOW_BASE+i, false);
    static_cast<wxMenuItemBase*>(m_apWindowMenuItems[i])->SetName (wxString("<Empty>"));
    m_apWindowMenuData[i] = NULL;
  }
  
}

void MainFrame::OnMRUFile(wxCommandEvent& event)
{
   wxString fileName (theApp->getDocManager()->GetHistoryFile(event.GetSelection()));
   if (fileName != "")
     theApp->getDocManager()->CreateDocument(fileName, wxDOC_SILENT);
}

void 
MainFrame::DoWindowMenu (int iMenuPosition, wxCommandEvent& event)
{
  if (wxDocument* pDoc = m_apWindowMenuData [iMenuPosition]) {
    wxString strFilename = pDoc->GetFilename();
    const wxView* pView = pDoc->GetFirstView();
    if (pView) {
      wxFrame* pFrame = pView->GetFrame();
      pFrame->SetFocus();
      pFrame->Raise();
    }
  }
}

void MainFrame::OnWindowMenu0 (wxCommandEvent& event)
{ DoWindowMenu (0, event); }

void MainFrame::OnWindowMenu1 (wxCommandEvent& event)
{ DoWindowMenu (1, event); }

void MainFrame::OnWindowMenu2 (wxCommandEvent& event)
{ DoWindowMenu (2, event); }

void MainFrame::OnWindowMenu3 (wxCommandEvent& event)
{ DoWindowMenu (3, event); }

void MainFrame::OnWindowMenu4 (wxCommandEvent& event)
{ DoWindowMenu (4, event); }

void MainFrame::OnWindowMenu5 (wxCommandEvent& event)
{ DoWindowMenu (5, event); }

void MainFrame::OnWindowMenu6 (wxCommandEvent& event)
{ DoWindowMenu (6, event); }

void MainFrame::OnWindowMenu7 (wxCommandEvent& event)
{ DoWindowMenu (7, event); }

void MainFrame::OnWindowMenu8 (wxCommandEvent& event)
{ DoWindowMenu (8, event); }

void MainFrame::OnWindowMenu9 (wxCommandEvent& event)
{ DoWindowMenu (9, event); }

void MainFrame::OnWindowMenu10 (wxCommandEvent& event)
{ DoWindowMenu (10, event); }

void MainFrame::OnWindowMenu11 (wxCommandEvent& event)
{ DoWindowMenu (11, event); }

void MainFrame::OnWindowMenu12 (wxCommandEvent& event)
{ DoWindowMenu (12, event); }

void MainFrame::OnWindowMenu13 (wxCommandEvent& event)
{ DoWindowMenu (13, event); }

void MainFrame::OnWindowMenu14 (wxCommandEvent& event)
{ DoWindowMenu (14, event); }

void MainFrame::OnWindowMenu15 (wxCommandEvent& event)
{ DoWindowMenu (15, event); }

void MainFrame::OnWindowMenu16 (wxCommandEvent& event)
{ DoWindowMenu (16, event); }

void MainFrame::OnWindowMenu17 (wxCommandEvent& event)
{ DoWindowMenu (17, event); }

void MainFrame::OnWindowMenu18 (wxCommandEvent& event)
{ DoWindowMenu (18, event); }

void MainFrame::OnWindowMenu19 (wxCommandEvent& event)
{ DoWindowMenu (19, event); }


class BitmapControl : public wxPanel
{
private:  
  DECLARE_DYNAMIC_CLASS (BitmapControl)
    DECLARE_EVENT_TABLE ()
    wxBitmap* m_pBitmap;
  
public:
  BitmapControl (wxBitmap* pBitmap, wxWindow *parent, wxWindowID id = -1,
    const wxPoint& pos = wxDefaultPosition,
    const wxSize& size = wxDefaultSize,
    long style = wxSTATIC_BORDER,
    const wxValidator& validator = wxDefaultValidator,
    const wxString& name = "BitmapCtrl");
  
  
  virtual ~BitmapControl();
  
  virtual wxSize GetBestSize() const;
  
  wxBitmap* getBitmap() 
  { return m_pBitmap; }
  
  void OnPaint(wxPaintEvent& event);
};


BEGIN_EVENT_TABLE(BitmapControl, wxPanel)
EVT_PAINT(BitmapControl::OnPaint)
END_EVENT_TABLE()

IMPLEMENT_CLASS(BitmapControl, wxPanel)


BitmapControl::BitmapControl (wxBitmap* pBitmap, wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, 
                              long style, const wxValidator& validator, const wxString& name)
                              : m_pBitmap(pBitmap)
{
  Create(parent, id, pos, size, style, name);
  
  SetSize (GetBestSize());
}

wxSize
BitmapControl::GetBestSize () const
{
  if (m_pBitmap)
    return wxSize (m_pBitmap->GetWidth(), m_pBitmap->GetHeight());
  else
    return wxSize(0,0);
}

BitmapControl::~BitmapControl()
{}

void
BitmapControl::OnPaint (wxPaintEvent& event)
{
  wxPaintDC dc(this);
  if (m_pBitmap)
    dc.DrawBitmap (*m_pBitmap, 0, 0);
}


class BitmapDialog : public wxDialog {
private:
  BitmapControl* m_pBitmapCtrl;
  
public:
  BitmapDialog (wxBitmap* pBitmap, char const* pszTitle);
  virtual ~BitmapDialog();
};

BitmapDialog::BitmapDialog (wxBitmap* pBitmap, char const* pszTitle)
: wxDialog(theApp->getMainFrame(), -1, wxString(pszTitle), wxDefaultPosition, wxDefaultSize, wxDIALOG_MODAL | wxDEFAULT_DIALOG_STYLE)
{
  wxBoxSizer* pTopSizer = new wxBoxSizer (wxVERTICAL);
  
  pTopSizer->Add (new BitmapControl (pBitmap, this), 0, wxALIGN_CENTER | wxALL, 5);
  
  wxBoxSizer* pButtonSizer = new wxBoxSizer (wxHORIZONTAL);
  wxButton* pButtonOk = new wxButton (this, wxID_OK, "Ok");
  pButtonSizer->Add (pButtonOk, 0, wxEXPAND | wxALL, 10);
  
  pTopSizer->Add (pButtonSizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout (true);
  SetSizer (pTopSizer);
  pTopSizer->Fit (this);
  pTopSizer->SetSizeHints (this);
}

BitmapDialog::~BitmapDialog()
{}


#include "./splash.xpm"
void 
MainFrame::OnAbout(wxCommandEvent& WXUNUSED(event) )
{
  wxBitmap bmp (splash);
  if (bmp.Ok()) {
    BitmapDialog dlg (&bmp, "About CTSim");
    dlg.Show(true);
  } else {
    wxString msg = "CTSim\nThe Open Source Computed Tomography Simulator\n";
#ifdef CTSIMVERSION
    msg += "Version ";
    msg += CTSIMVERSION;
    msg += "\n\n";
#elif defined(VERSION)
    msg << "Version: " <<  VERSION << "\n\n";
#endif
    msg += "Author: Kevin Rosenberg <kevin@rosenberg.net>\nUsage: ctsim [files-to-open..] [--help]";
    
    wxMessageBox(msg, "About CTSim", wxOK | wxICON_INFORMATION, this);
  }
}

