/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:          dialogs.h
**   Purpose:       Header file for Dialogs of CTSim program
**   Programmer:    Kevin Rosenberg
**   Date Started:  July 2000
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: dialogs.h,v 1.20 2001/01/04 21:28:41 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/


#ifndef __DIALOGSH__
#define __DIALOGSH__

#include "wx/wx.h"
#include <string>
#include "ctsupport.h"
#include "scanner.h"
#include "phantom.h"
#include "procsignal.h"
#include "filter.h"
#include "projections.h"


// CLASS StringValueAndTitleListBox
//
// A superclass of wxListBox that can handle string values and titles
// and by displaying the title in the list box and returning the string value

class StringValueAndTitleListBox : public wxListBox
{
 public:
  StringValueAndTitleListBox (wxDialog* pParent, int nChoices, const char* aszTitle[], const char* aszValue[]);

    const char* getSelectionStringValue () const;

 private:
    const char** m_ppszValues;
};


class DialogGetPhantom : public wxDialog
{
 public:
    DialogGetPhantom (wxFrame* pParent, int iDefaultPhantom = Phantom::PHM_HERMAN);
    virtual ~DialogGetPhantom () {}

    const char* getPhantom ();

 private:
    StringValueAndTitleListBox* m_pListBoxPhantom;
};


#include <vector>
class ImageFileDocument;
class DialogGetComparisonImage : public wxDialog
{
 public:
   DialogGetComparisonImage (wxFrame* pParent, const char* const pszTitle, const std::vector<ImageFileDocument*>& rVecIF, bool bShowMakeDifference);
    virtual ~DialogGetComparisonImage () {}

    ImageFileDocument* getImageFileDocument ();

    bool getMakeDifferenceImage();

 private:
    wxListBox* m_pListBoxImageChoices;
    wxCheckBox* m_pMakeDifferenceImage;
    const std::vector<ImageFileDocument*>& m_rVecIF;
};


class ImageFile;
class DialogGetMinMax : public wxDialog
{
 public:
    DialogGetMinMax (wxFrame* pParent, const char* const pszTitle, double dDefaultMin = 0., double dDefaultMax = 0.);
    virtual ~DialogGetMinMax ();

    double getMinimum ();
    double getMaximum ();

 private:
    wxTextCtrl* m_pTextCtrlMin;
    wxTextCtrl* m_pTextCtrlMax;

    double m_dDefaultMin;
    double m_dDefaultMax;
};


class DialogGetRasterParameters : public wxDialog
{
 public:
    DialogGetRasterParameters (wxFrame* pParent, int iDefaultXSize = 0, int iDefaultYSize = 0, int iDefaultNSamples = 1);
    virtual ~DialogGetRasterParameters ();

    unsigned int getXSize ();
    unsigned int getYSize ();
    unsigned int getNSamples ();

 private:
    wxTextCtrl* m_pTextCtrlXSize;
    wxTextCtrl* m_pTextCtrlYSize;
    wxTextCtrl* m_pTextCtrlNSamples;

    int m_iDefaultXSize;
    int m_iDefaultYSize;
    int m_iDefaultNSamples;
};


class DialogGetProjectionParameters : public wxDialog
{
 public:
    DialogGetProjectionParameters (wxFrame* pParent, int iDefaultNDet = 0, int iDefaultNView = 0, int iDefaultNSamples = 1, double dDefaultRotAngle = 1., double dDefaultFocalLength = 1, double dDefaultFieldOfView = 1., int iDefaultGeometry = Scanner::GEOMETRY_PARALLEL, int iDefaultTrace = Trace::TRACE_NONE);
    ~DialogGetProjectionParameters ();

    unsigned int getNDet ();
    unsigned int getNView ();
    unsigned int getNSamples ();
    int getTrace ();

    double getRotAngle ();
    double getFieldOfViewRatio ();
    double getFocalLengthRatio ();
    const char* getGeometry();

 private:
    wxTextCtrl* m_pTextCtrlNDet;
    wxTextCtrl* m_pTextCtrlNView;
    wxTextCtrl* m_pTextCtrlNSamples;
    wxTextCtrl* m_pTextCtrlRotAngle;
    wxTextCtrl* m_pTextCtrlFocalLength;
    wxTextCtrl* m_pTextCtrlFieldOfView;

    StringValueAndTitleListBox* m_pListBoxGeometry;
    StringValueAndTitleListBox* m_pListBoxTrace;

    int m_iDefaultNDet;
    int m_iDefaultNView;
    int m_iDefaultNSamples;
    int m_iDefaultTrace;
    int m_iDefaultGeometry;
    double m_dDefaultRotAngle;
    double m_dDefaultFocalLength;
    double m_dDefaultFieldOfView;
};


#include "backprojectors.h"
class DialogGetReconstructionParameters : public wxDialog
{
 public:
    DialogGetReconstructionParameters (wxFrame* pParent, int iDefaultXSize = 0, int iDefaultYSize = 0, int iDefaultFilterID = SignalFilter::FILTER_ABS_BANDLIMIT, double dDefaultFilterParam = 1., int iDefaultFilterMethodID = ProcessSignal::FILTER_METHOD_CONVOLUTION, int iDefaultFilterGeneration = ProcessSignal::FILTER_GENERATION_INVALID, int iDefaultZeropad = 3, int iDefaultInterpID = Backprojector::INTERP_LINEAR, int iDefaultInterpParam = 1, int iDefaultBackprojectID = Backprojector::BPROJ_IDIFF3, int iDefaultTrace = Trace::TRACE_NONE);
    virtual ~DialogGetReconstructionParameters ();

    unsigned int getXSize();
    unsigned int getYSize();
    const char* getFilterName();
    double getFilterParam();
    const char* getFilterMethodName();
    unsigned int getZeropad();
    const char* getFilterGenerationName();
    const char* getInterpName();
    unsigned int getInterpParam();
    const char* getBackprojectName();
    int getTrace ();

 private:
    wxTextCtrl* m_pTextCtrlXSize;
    wxTextCtrl* m_pTextCtrlYSize;
    wxTextCtrl* m_pTextCtrlZeropad;
    wxTextCtrl* m_pTextCtrlFilterParam;
    wxTextCtrl* m_pTextCtrlInterpParam;

    StringValueAndTitleListBox* m_pListBoxFilter;
    StringValueAndTitleListBox* m_pListBoxFilterMethod;
    StringValueAndTitleListBox* m_pListBoxFilterGeneration;
    StringValueAndTitleListBox* m_pListBoxInterp;
    StringValueAndTitleListBox* m_pListBoxBackproject;
    StringValueAndTitleListBox* m_pListBoxTrace;

    int m_iDefaultXSize;
    int m_iDefaultYSize;
    double m_dDefaultFilterParam;
    int m_iDefaultZeropad;
    int m_iDefaultInterpParam;
    int m_iDefaultTrace;
};


class DialogGetFilterParameters : public wxDialog
{
 public:
    DialogGetFilterParameters (wxFrame* pParent, int iDefaultXSize = 0, int iDefaultYSize = 0, int iDefaultFilterID = SignalFilter::FILTER_BANDLIMIT, double dDefaultFilterParam = 1., double dDefaultBandwidth = 1., int iDefaultDomain = SignalFilter::DOMAIN_SPATIAL, double dDefaultInputScale = 1., double dDefaultOutputScale = 1.);
    virtual ~DialogGetFilterParameters ();

    unsigned int getXSize();
    unsigned int getYSize();
    const char* getFilterName();
    const char* getDomainName();
    double getFilterParam();
    double getInputScale();
    double getOutputScale();
    double getBandwidth();

 private:
    wxTextCtrl* m_pTextCtrlXSize;
    wxTextCtrl* m_pTextCtrlYSize;
    wxTextCtrl* m_pTextCtrlFilterParam;
    wxTextCtrl* m_pTextCtrlOutputScale;
    wxTextCtrl* m_pTextCtrlInputScale;
    wxTextCtrl* m_pTextCtrlBandwidth;

    StringValueAndTitleListBox* m_pListBoxFilter;
    StringValueAndTitleListBox* m_pListBoxDomain;

    int m_iDefaultXSize;
    int m_iDefaultYSize;
    double m_dDefaultFilterParam;
    double m_dDefaultBandwidth;
    double m_dDefaultOutputScale;
    double m_dDefaultInputScale;
    int m_iDefaultDomain;
};

class DialogExportParameters : public wxDialog
{
 public:
    DialogExportParameters (wxFrame* pParent, int iDefaultFormatID);
    virtual ~DialogExportParameters () {}

    const char* getFormatName();

 private:
    StringValueAndTitleListBox* m_pListBoxFormat;
};

class DialogAutoScaleParameters : public wxDialog
{
 public:
    DialogAutoScaleParameters (wxFrame* pParent, double mean, double mode, double median, double stddev, double dDefaultScaleFactor = 1.);
    virtual ~DialogAutoScaleParameters() {}

    bool getMinMax (double* pMin, double* pMax);
    double getAutoScaleFactor ();

 private:
    const double m_dMean;
	const double m_dMode;
	const double m_dMedian;
	const double m_dStdDev;

    wxTextCtrl* m_pTextCtrlStdDevFactor;
    wxListBox* m_pListBoxCenter;
};

class DialogGetXYSize : public wxDialog
{
 public:
    DialogGetXYSize (wxFrame* pParent, const char* const pszTitle, int iDefaultXSize = 1, int iDefaultYSize = 1);
    virtual ~DialogGetXYSize ();

    unsigned int getXSize ();
    unsigned int getYSize ();

 private:
    wxTextCtrl* m_pTextCtrlXSize;
    wxTextCtrl* m_pTextCtrlYSize;

    unsigned int m_iDefaultXSize;
    unsigned int m_iDefaultYSize;
};


class DialogGetConvertPolarParameters : public wxDialog
{
 public:
   DialogGetConvertPolarParameters (wxFrame* pParent, const char* const pszTitle, int iDefaultXSize = 0, 
     int iDefaultYSize = 0, int iDefaultInterpolationID = Projections::POLAR_INTERP_BILINEAR, 
     int iDefaultZeropad = 1);
   virtual ~DialogGetConvertPolarParameters ();

    unsigned int getXSize();
    unsigned int getYSize();
    const char* getInterpolationName();
    unsigned int getZeropad();

 private:
    wxTextCtrl* m_pTextCtrlXSize;
    wxTextCtrl* m_pTextCtrlYSize;
    wxTextCtrl* m_pTextCtrlZeropad;

    StringValueAndTitleListBox* m_pListBoxInterpolation;

    int m_iDefaultXSize;
    int m_iDefaultYSize;
    int m_iDefaultZeropad;
};


#endif

