/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:          view.cpp
**   Purpose:       View & Canvas routines for CTSim program
**   Programmer:    Kevin Rosenberg
**   Date Started:  July 2000
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: views.cpp,v 1.68 2001/01/20 08:10:33 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#if !wxUSE_DOC_VIEW_ARCHITECTURE
#error You must set wxUSE_DOC_VIEW_ARCHITECTURE to 1 in setup.h!
#endif

#include "wx/image.h"
#include "wx/progdlg.h"

#include "ct.h"
#include "ctsim.h"
#include "docs.h"
#include "views.h"
#include "dialogs.h"
#include "dlgprojections.h"
#include "dlgreconstruct.h"
#include "backprojectors.h"
#include "reconstruct.h"
#include "timer.h"

#if defined(MSVC) || HAVE_SSTREAM
#include <sstream>
#else
#include <sstream_subst>
#endif


// ImageFileCanvas

BEGIN_EVENT_TABLE(ImageFileCanvas, wxScrolledWindow)
EVT_MOUSE_EVENTS(ImageFileCanvas::OnMouseEvent)
END_EVENT_TABLE()


ImageFileCanvas::ImageFileCanvas (ImageFileView* v, wxFrame *frame, const wxPoint& pos, const wxSize& size, const long style)
: wxScrolledWindow(frame, -1, pos, size, style)
{
  m_pView = v;
  m_xCursor = -1;
  m_yCursor = -1;
}

void 
ImageFileCanvas::OnDraw(wxDC& dc)
{
  if (m_pView)
    m_pView->OnDraw(& dc);
}

void 
ImageFileCanvas::DrawRubberBandCursor (wxDC& dc, int x, int y)
{
  const ImageFile& rIF = m_pView->GetDocument()->getImageFile();
  int nx = rIF.nx();
  int ny = rIF.ny();
  
  int yPt = ny - y - 1;
  dc.SetLogicalFunction (wxINVERT);
  dc.SetPen (*wxGREEN_PEN);
  dc.DrawLine (0, yPt, nx, yPt);
  dc.DrawLine (x, 0, x, ny);
  dc.SetLogicalFunction (wxCOPY);
}

bool
ImageFileCanvas::GetCurrentCursor (int& x, int& y)
{
  x = m_xCursor;
  y = m_yCursor;
  
  if (m_xCursor >= 0 && m_yCursor >= 0)
    return true;
  else
    return false;
}

void 
ImageFileCanvas::OnMouseEvent(wxMouseEvent& event)
{
  if (! m_pView)
    return;
  
  wxClientDC dc(this);
  PrepareDC(dc);
  
  wxPoint pt(event.GetLogicalPosition(dc));
  
  const ImageFileDocument* pIFDoc = m_pView->GetDocument();
  if (! pIFDoc)
    return;
  const ImageFile& rIF = pIFDoc->getImageFile();
  ImageFileArrayConst v = rIF.getArray();
  int nx = rIF.nx();
  int ny = rIF.ny();
  const int yPt = ny - 1 - pt.y;
  if (event.RightIsDown()) {
    if (pt.x >= 0 && pt.x < nx && pt.y >= 0 && pt.y < ny) {
      std::ostringstream os;
      os << "Image value (" << pt.x << "," << yPt << ") = " << v[pt.x][yPt];
      if (rIF.isComplex()) {
        double dImag = rIF.getImaginaryArray()[pt.x][yPt];
        if (dImag < 0)
          os << " - " << -dImag;
        else
          os << " + " << dImag;
        os << "i\n";
      } else
        os << "\n";
      *theApp->getLog() << os.str().c_str();
    } else
      *theApp->getLog() << "Mouse out of image range (" << pt.x << "," << yPt << ")\n";
  }
  else if (event.LeftIsDown() || event.LeftUp() || event.RightUp()) {
    if (pt.x >= 0 && pt.x < nx && pt.y >= 0 && pt.y < ny) {
      if (m_xCursor >= 0 && m_yCursor >= 0) {
        DrawRubberBandCursor (dc, m_xCursor, m_yCursor);
      }
      DrawRubberBandCursor (dc, pt.x, yPt);
      m_xCursor = pt.x;
      m_yCursor = yPt;
    } else
      *theApp->getLog() << "Mouse out of image range (" << pt.x << "," << yPt << ")\n";
  }
  if (event.LeftUp()) {
    std::ostringstream os;
    os << "Selected column " << pt.x << " , row " << yPt << "\n";
    *theApp->getLog() << os.str().c_str();
  }
}

wxSize
ImageFileCanvas::GetBestSize() const
{
  if (! m_pView)
    return wxSize(0,0);
  
  const ImageFile& rIF = m_pView->GetDocument()->getImageFile();
  return wxSize (rIF.nx(), rIF.ny());
}


// ImageFileView

IMPLEMENT_DYNAMIC_CLASS(ImageFileView, wxView)

BEGIN_EVENT_TABLE(ImageFileView, wxView)
EVT_MENU(IFMENU_FILE_EXPORT, ImageFileView::OnExport)
EVT_MENU(IFMENU_FILE_PROPERTIES, ImageFileView::OnProperties)
EVT_MENU(IFMENU_VIEW_SCALE_MINMAX, ImageFileView::OnScaleMinMax)
EVT_MENU(IFMENU_VIEW_SCALE_AUTO, ImageFileView::OnScaleAuto)
EVT_MENU(IFMENU_VIEW_SCALE_FULL, ImageFileView::OnScaleFull)
EVT_MENU(IFMENU_COMPARE_IMAGES, ImageFileView::OnCompare)
EVT_MENU(IFMENU_COMPARE_ROW, ImageFileView::OnCompareRow)
EVT_MENU(IFMENU_COMPARE_COL, ImageFileView::OnCompareCol)
EVT_MENU(IFMENU_FILTER_INVERTVALUES, ImageFileView::OnInvertValues)
EVT_MENU(IFMENU_FILTER_SQUARE, ImageFileView::OnSquare)
EVT_MENU(IFMENU_FILTER_SQRT, ImageFileView::OnSquareRoot)
EVT_MENU(IFMENU_FILTER_LOG, ImageFileView::OnLog)
EVT_MENU(IFMENU_FILTER_EXP, ImageFileView::OnExp)
EVT_MENU(IFMENU_FILTER_FOURIER, ImageFileView::OnFourier)
EVT_MENU(IFMENU_FILTER_INVERSE_FOURIER, ImageFileView::OnInverseFourier)
EVT_MENU(IFMENU_FILTER_SHUFFLEFOURIERTONATURALORDER, ImageFileView::OnShuffleFourierToNaturalOrder)
EVT_MENU(IFMENU_FILTER_SHUFFLENATURALTOFOURIERORDER, ImageFileView::OnShuffleNaturalToFourierOrder)
EVT_MENU(IFMENU_IMAGE_ADD, ImageFileView::OnAdd)
EVT_MENU(IFMENU_IMAGE_SUBTRACT, ImageFileView::OnSubtract)
EVT_MENU(IFMENU_IMAGE_MULTIPLY, ImageFileView::OnMultiply)
EVT_MENU(IFMENU_IMAGE_DIVIDE, ImageFileView::OnDivide)
EVT_MENU(IFMENU_IMAGE_SCALESIZE, ImageFileView::OnScaleSize)
#ifdef HAVE_FFT
EVT_MENU(IFMENU_FILTER_FFT, ImageFileView::OnFFT)
EVT_MENU(IFMENU_FILTER_IFFT, ImageFileView::OnIFFT)
EVT_MENU(IFMENU_FILTER_FFT_ROWS, ImageFileView::OnFFTRows)
EVT_MENU(IFMENU_FILTER_IFFT_ROWS, ImageFileView::OnIFFTRows)
EVT_MENU(IFMENU_FILTER_FFT_COLS, ImageFileView::OnFFTCols)
EVT_MENU(IFMENU_FILTER_IFFT_COLS, ImageFileView::OnIFFTCols)
#endif
EVT_MENU(IFMENU_FILTER_MAGNITUDE, ImageFileView::OnMagnitude)
EVT_MENU(IFMENU_FILTER_PHASE, ImageFileView::OnPhase)
EVT_MENU(IFMENU_PLOT_ROW, ImageFileView::OnPlotRow)
EVT_MENU(IFMENU_PLOT_COL, ImageFileView::OnPlotCol)
#ifdef HAVE_FFT
EVT_MENU(IFMENU_PLOT_FFT_ROW, ImageFileView::OnPlotFFTRow)
EVT_MENU(IFMENU_PLOT_FFT_COL, ImageFileView::OnPlotFFTCol)
#endif
EVT_MENU(IFMENU_PLOT_HISTOGRAM, ImageFileView::OnPlotHistogram)
END_EVENT_TABLE()

ImageFileView::ImageFileView(void) 
: wxView(), m_canvas(NULL), m_frame(NULL), m_bMinSpecified(false), m_bMaxSpecified(false)
{
  m_iDefaultExportFormatID = ImageFile::FORMAT_PNG;
}

ImageFileView::~ImageFileView(void)
{
}

void
ImageFileView::OnProperties (wxCommandEvent& event)
{
  const ImageFile& rIF = GetDocument()->getImageFile();
  if (rIF.nx() == 0 || rIF.ny() == 0)
    *theApp->getLog() << "Properties: empty imagefile\n";
  else {
    const std::string& rFilename = rIF.getFilename();
    std::ostringstream os;
    double min, max, mean, mode, median, stddev;
    rIF.statistics (rIF.getArray(), min, max, mean, mode, median, stddev);
    os << "Filename: " << rFilename << "\n";
    os << "Size: (" << rIF.nx() << "," << rIF.ny() << ")\n";
    os << "Data type: ";
    if (rIF.isComplex())
      os << "Complex\n";
    else
      os << "Real\n";
    os << "\nMinimum: "<<min<<"\nMaximum: "<<max<<"\nMean: "<<mean<<"\nMedian: "<<median<<"\nMode: "<<mode<<"\nStandard Deviation: "<<stddev << "\n";
    if (rIF.isComplex()) {
      rIF.statistics (rIF.getImaginaryArray(), min, max, mean, mode, median, stddev);
      os << "\nImaginary: min: "<<min<<"\nmax: "<<max<<"\nmean: "<<mean<<"\nmedian: "<<median<<"\nmode: "<<mode<<"\nstddev: "<<stddev << "\n";
    }
    if (rIF.nLabels() > 0) {
      os << "\n";
      rIF.printLabelsBrief (os);
    }
    *theApp->getLog() << os.str().c_str();
    wxMessageDialog dialogMsg (getFrameForChild(), os.str().c_str(), "Imagefile Properties", wxOK | wxICON_INFORMATION);
    dialogMsg.ShowModal();
  }
}

void 
ImageFileView::OnScaleAuto (wxCommandEvent& event)
{
  const ImageFile& rIF = GetDocument()->getImageFile();
  double min, max, mean, mode, median, stddev;
  rIF.statistics(min, max, mean, mode, median, stddev);
  DialogAutoScaleParameters dialogAutoScale (getFrameForChild(), mean, mode, median, stddev, m_dAutoScaleFactor);
  int iRetVal = dialogAutoScale.ShowModal();
  if (iRetVal == wxID_OK) {
    m_bMinSpecified = true;
    m_bMaxSpecified = true;
    double dMin, dMax;
    if (dialogAutoScale.getMinMax (&dMin, &dMax)) {
      m_dMinPixel = dMin;
      m_dMaxPixel = dMax;
      m_dAutoScaleFactor = dialogAutoScale.getAutoScaleFactor();
      OnUpdate (this, NULL);
    }
  }
}

void 
ImageFileView::OnScaleMinMax (wxCommandEvent& event)
{
  const ImageFile& rIF = GetDocument()->getImageFile();
  double min, max;
  if (! m_bMinSpecified && ! m_bMaxSpecified)
    rIF.getMinMax (min, max);
  
  if (m_bMinSpecified)
    min = m_dMinPixel;
  if (m_bMaxSpecified)
    max = m_dMaxPixel;
  
  DialogGetMinMax dialogMinMax (getFrameForChild(), "Set Image Minimum & Maximum", min, max);
  int retVal = dialogMinMax.ShowModal();
  if (retVal == wxID_OK) {
    m_bMinSpecified = true;
    m_bMaxSpecified = true;
    m_dMinPixel = dialogMinMax.getMinimum();
    m_dMaxPixel = dialogMinMax.getMaximum();
    OnUpdate (this, NULL);
  }
}

void 
ImageFileView::OnScaleFull (wxCommandEvent& event)
{
  if (m_bMinSpecified || m_bMaxSpecified) {
    m_bMinSpecified = false;
    m_bMaxSpecified = false;
    OnUpdate (this, NULL);
  }
}

void
ImageFileView::OnCompare (wxCommandEvent& event)
{
  std::vector<ImageFileDocument*> vecIF;
  theApp->getCompatibleImages (GetDocument(), vecIF);
  
  if (vecIF.size() == 0) {
    wxMessageBox("There are no compatible image files open for comparision", "No comparison images");
  } else {
    DialogGetComparisonImage dialogGetCompare(getFrameForChild(), "Get Comparison Image", vecIF, true);
    
    if (dialogGetCompare.ShowModal() == wxID_OK) {
      const ImageFile& rIF = GetDocument()->getImageFile();
      ImageFileDocument* pCompareDoc = dialogGetCompare.getImageFileDocument();
      const ImageFile& rCompareIF = pCompareDoc->getImageFile();
      std::ostringstream os;
      double min, max, mean, mode, median, stddev;
      rIF.statistics (min, max, mean, mode, median, stddev);
      os << GetFrame()->GetTitle().c_str() << ": minimum=" << min << ", maximum=" << max << ", mean=" << mean << ", mode=" << mode << ", median=" << median << ", stddev=" << stddev << "\n";
      rCompareIF.statistics (min, max, mean, mode, median, stddev);
      os << pCompareDoc->GetFirstView()->GetFrame()->GetTitle().c_str() << ": minimum=" << min << ", maximum=" << max << ", mean=" << mean << ", mode=" << mode << ", median=" << median << ", stddev=" << stddev << "\n";
      os << "\n";
      double d, r, e;
      rIF.comparativeStatistics (rCompareIF, d, r, e);
      os << "Comparative Statistics: d=" << d << ", r=" << r << ", e=" << e << "\n";
      *theApp->getLog() << os.str().c_str();
      if (dialogGetCompare.getMakeDifferenceImage()) {
        ImageFileDocument* pDifferenceDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
        if (! pDifferenceDoc) {
          sys_error (ERR_SEVERE, "Unable to create image file");
          return;
        }
        ImageFile& differenceImage = pDifferenceDoc->getImageFile();
        
        differenceImage.setArraySize (rIF.nx(), rIF.ny());
        if (! rIF.subtractImages (rCompareIF, differenceImage)) {
          pDifferenceDoc->DeleteAllViews();
          return;
        }
        
        wxString s = GetFrame()->GetTitle() + ": ";
        differenceImage.labelsCopy (rIF, s.c_str());
        s = pCompareDoc->GetFirstView()->GetFrame()->GetTitle() + ": ";
        differenceImage.labelsCopy (rCompareIF, s.c_str());
        std::ostringstream osLabel;
        osLabel << "Compare image " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str() 
          << " and " << pCompareDoc->GetFirstView()->GetFrame()->GetTitle().c_str() << ": "
          << os.str().c_str();
        differenceImage.labelAdd (os.str().c_str());
        if (theApp->getSetModifyNewDocs())
          pDifferenceDoc->Modify(true);
        pDifferenceDoc->UpdateAllViews(this);
        pDifferenceDoc->GetFirstView()->OnUpdate (this, NULL);
      }
      wxMessageBox(os.str().c_str(), "Image Comparison");
    }
  }
}

void
ImageFileView::OnInvertValues (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.invertPixelValues (rIF);
  rIF.labelAdd ("Invert Pixel Values");
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnSquare (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.square (rIF);
  rIF.labelAdd ("Square Pixel Values");
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnSquareRoot (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.sqrt (rIF);
  rIF.labelAdd ("Square-root Pixel Values");
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnLog (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.log (rIF);
  rIF.labelAdd ("Logrithm base-e Pixel Values");
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnExp (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.exp (rIF);
  rIF.labelAdd ("Exponent base-e Pixel Values");
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnAdd (wxCommandEvent& event)
{
  std::vector<ImageFileDocument*> vecIF;
  theApp->getCompatibleImages (GetDocument(), vecIF);
  
  if (vecIF.size() == 0) {
    wxMessageBox ("There are no compatible image files open for comparision", "No comparison images");
  } else {
    DialogGetComparisonImage dialogGetCompare (getFrameForChild(), "Get Image to Add", vecIF, false);
    
    if (dialogGetCompare.ShowModal() == wxID_OK) {
      ImageFile& rIF = GetDocument()->getImageFile();
      ImageFileDocument* pRHSDoc = dialogGetCompare.getImageFileDocument();
      const ImageFile& rRHSIF = pRHSDoc->getImageFile();
      ImageFileDocument* pNewDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
        if (! pNewDoc) {
          sys_error (ERR_SEVERE, "Unable to create image file");
          return;
        }
      ImageFile& newImage = pNewDoc->getImageFile();  
      newImage.setArraySize (rIF.nx(), rIF.ny());
      rIF.addImages (rRHSIF, newImage);
      std::ostringstream os;
      os << "Add image " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str() << " and " 
        << pRHSDoc->GetFirstView()->GetFrame()->GetTitle().c_str();
      wxString s = GetDocument()->GetFirstView()->GetFrame()->GetTitle() + ": ";
      newImage.labelsCopy (rIF, s.c_str());
      s = pRHSDoc->GetFirstView()->GetFrame()->GetTitle() + ": ";
      newImage.labelsCopy (rRHSIF, s.c_str());
      newImage.labelAdd (os.str().c_str());
      *theApp->getLog() << os.str().c_str() << "\n";
      if (theApp->getSetModifyNewDocs())
        pNewDoc->Modify(TRUE);
      pNewDoc->UpdateAllViews(this);
      pNewDoc->GetFirstView()->OnUpdate (this, NULL);
    }
  }
}

void
ImageFileView::OnSubtract (wxCommandEvent& event)
{
  std::vector<ImageFileDocument*> vecIF;
  theApp->getCompatibleImages (GetDocument(), vecIF);
  
  if (vecIF.size() == 0) {
    wxMessageBox ("There are no compatible image files open for comparision", "No comparison images");
  } else {
    DialogGetComparisonImage dialogGetCompare (getFrameForChild(), "Get Image to Subtract", vecIF, false);
    
    if (dialogGetCompare.ShowModal() == wxID_OK) {
      ImageFile& rIF = GetDocument()->getImageFile();
      ImageFileDocument* pRHSDoc = dialogGetCompare.getImageFileDocument();
      const ImageFile& rRHSIF = pRHSDoc->getImageFile();
      ImageFileDocument* pNewDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
        if (! pNewDoc) {
          sys_error (ERR_SEVERE, "Unable to create image file");
          return;
        }
      ImageFile& newImage = pNewDoc->getImageFile();  
      newImage.setArraySize (rIF.nx(), rIF.ny());
      rIF.subtractImages (rRHSIF, newImage);
      std::ostringstream os;
      os << "Subtract image " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str() << " and " 
        << pRHSDoc->GetFirstView()->GetFrame()->GetTitle().c_str();
      wxString s = GetDocument()->GetFirstView()->GetFrame()->GetTitle() + ": ";
      newImage.labelsCopy (rIF, s.c_str());
      s = pRHSDoc->GetFirstView()->GetFrame()->GetTitle() + ": ";
      newImage.labelsCopy (rRHSIF, s.c_str());
      newImage.labelAdd (os.str().c_str());
      *theApp->getLog() << os.str().c_str() << "\n";
      if (theApp->getSetModifyNewDocs())
        pNewDoc->Modify(TRUE);
      pNewDoc->UpdateAllViews(this);
      pNewDoc->GetFirstView()->OnUpdate (this, NULL);
    }
  }
}

void
ImageFileView::OnMultiply (wxCommandEvent& event)
{
  std::vector<ImageFileDocument*> vecIF;
  theApp->getCompatibleImages (GetDocument(), vecIF);
  
  if (vecIF.size() == 0) {
    wxMessageBox ("There are no compatible image files open for comparision", "No comparison images");
  } else {
    DialogGetComparisonImage dialogGetCompare (getFrameForChild(), "Get Image to Multiply", vecIF, false);
    
    if (dialogGetCompare.ShowModal() == wxID_OK) {
      ImageFile& rIF = GetDocument()->getImageFile();
      ImageFileDocument* pRHSDoc = dialogGetCompare.getImageFileDocument();
      const ImageFile& rRHSIF = pRHSDoc->getImageFile();
      ImageFileDocument* pNewDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
        if (! pNewDoc) {
          sys_error (ERR_SEVERE, "Unable to create image file");
          return;
        }
      ImageFile& newImage = pNewDoc->getImageFile();  
      newImage.setArraySize (rIF.nx(), rIF.ny());
      rIF.multiplyImages (rRHSIF, newImage);
      std::ostringstream os;
      os << "Multiply image " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str() << " and " 
        << pRHSDoc->GetFirstView()->GetFrame()->GetTitle().c_str();
      wxString s = GetDocument()->GetFirstView()->GetFrame()->GetTitle() + ": ";
      newImage.labelsCopy (rIF, s.c_str());
      s = pRHSDoc->GetFirstView()->GetFrame()->GetTitle() + ": ";
      newImage.labelsCopy (rRHSIF, s.c_str());
      newImage.labelAdd (os.str().c_str());
      *theApp->getLog() << os.str().c_str() << "\n";
      if (theApp->getSetModifyNewDocs())
        pNewDoc->Modify(TRUE);
      pNewDoc->UpdateAllViews(this);
      pNewDoc->GetFirstView()->OnUpdate (this, NULL);
    }
  }
}

void
ImageFileView::OnDivide (wxCommandEvent& event)
{
  std::vector<ImageFileDocument*> vecIF;
  theApp->getCompatibleImages (GetDocument(), vecIF);
  
  if (vecIF.size() == 0) {
    wxMessageBox ("There are no compatible image files open for comparision", "No comparison images");
  } else {
    DialogGetComparisonImage dialogGetCompare (getFrameForChild(), "Get Image to Divide", vecIF, false);
    
    if (dialogGetCompare.ShowModal() == wxID_OK) {
      ImageFile& rIF = GetDocument()->getImageFile();
      ImageFileDocument* pRHSDoc = dialogGetCompare.getImageFileDocument();
      const ImageFile& rRHSIF = pRHSDoc->getImageFile();
      ImageFileDocument* pNewDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
        if (! pNewDoc) {
          sys_error (ERR_SEVERE, "Unable to create image file");
          return;
        }
      ImageFile& newImage = pNewDoc->getImageFile();  
      newImage.setArraySize (rIF.nx(), rIF.ny());
      rIF.divideImages (rRHSIF, newImage);
      std::ostringstream os;
      os << "Divide image " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str() << " by " 
        << pRHSDoc->GetFirstView()->GetFrame()->GetTitle().c_str();
      wxString s = GetDocument()->GetFirstView()->GetFrame()->GetTitle() + ": ";
      newImage.labelsCopy (rIF, s.c_str());
      s = pRHSDoc->GetFirstView()->GetFrame()->GetTitle() + ": ";
      newImage.labelsCopy (rRHSIF, s.c_str());
      newImage.labelAdd (os.str().c_str());
      *theApp->getLog() << os.str().c_str() << "\n";
      if (theApp->getSetModifyNewDocs())
        pNewDoc->Modify(TRUE);
      pNewDoc->UpdateAllViews(this);
      pNewDoc->GetFirstView()->OnUpdate (this, NULL);
    }
  }
}


#ifdef HAVE_FFT
void
ImageFileView::OnFFT (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.fft (rIF);
  rIF.labelAdd ("FFT Image");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnIFFT (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.ifft (rIF);
  rIF.labelAdd ("IFFT Image");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnFFTRows (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.fftRows (rIF);
  rIF.labelAdd ("FFT Rows");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnIFFTRows (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.ifftRows (rIF);
  rIF.labelAdd ("IFFT Rows");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnFFTCols (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.fftCols (rIF);
  rIF.labelAdd ("FFT Columns");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnIFFTCols (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  rIF.ifftCols (rIF);
  rIF.labelAdd ("IFFT Columns");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}
#endif

void
ImageFileView::OnFourier (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  wxProgressDialog dlgProgress (wxString("Fourier"), wxString("Fourier Progress"), 1, getFrameForChild(), wxPD_APP_MODAL);
  rIF.fourier (rIF);
  rIF.labelAdd ("Fourier Image");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnInverseFourier (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  wxProgressDialog dlgProgress (wxString("Inverse Fourier"), wxString("Inverse Fourier Progress"), 1, getFrameForChild(), wxPD_APP_MODAL);
  rIF.inverseFourier (rIF);
  rIF.labelAdd ("Inverse Fourier Image");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnShuffleNaturalToFourierOrder (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  Fourier::shuffleNaturalToFourierOrder (rIF);
  rIF.labelAdd ("Shuffle Natural To Fourier Order");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnShuffleFourierToNaturalOrder (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  Fourier::shuffleFourierToNaturalOrder (rIF);
  rIF.labelAdd ("Shuffle Fourier To Natural Order");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
}

void
ImageFileView::OnMagnitude (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  if (rIF.isComplex()) {
    rIF.magnitude (rIF);
    rIF.labelAdd ("Magnitude of complex-image");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
  }
}

void
ImageFileView::OnPhase (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  if (rIF.isComplex()) {
    rIF.phase (rIF);
    rIF.labelAdd ("Phase of complex-image");
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  if (theApp->getSetModifyNewDocs())
    GetDocument()->Modify(TRUE);
  GetDocument()->UpdateAllViews(this);
  }
}


ImageFileCanvas* 
ImageFileView::CreateCanvas (wxView *view, wxFrame *parent)
{
  ImageFileCanvas* pCanvas;
  int width, height;
  parent->GetClientSize(&width, &height);
  
  pCanvas = new ImageFileCanvas (dynamic_cast<ImageFileView*>(view), parent, wxPoint(0, 0), wxSize(width, height), 0);
  
  pCanvas->SetScrollbars(20, 20, 50, 50);
  pCanvas->SetBackgroundColour(*wxWHITE);
  pCanvas->Clear();
  
  return pCanvas;
}

wxFrame*
ImageFileView::CreateChildFrame(wxDocument *doc, wxView *view)
{
#if CTSIM_MDI
  wxDocMDIChildFrame* subframe = new wxDocMDIChildFrame (doc, view, theApp->getMainFrame(), -1, "ImageFile Frame", wxPoint(-1, -1), wxSize(0, 0), wxDEFAULT_FRAME_STYLE);
#else
  wxDocChildFrame* subframe = new wxDocChildFrame (doc, view, theApp->getMainFrame(), -1, "ImageFile Frame", wxPoint(-1, -1), wxSize(0, 0), wxDEFAULT_FRAME_STYLE);
#endif
  theApp->setIconForFrame (subframe);

  wxMenu *file_menu = new wxMenu;
  
  file_menu->Append(MAINMENU_FILE_CREATE_PHANTOM, "Cr&eate Phantom...\tCtrl-P");
  file_menu->Append(MAINMENU_FILE_CREATE_FILTER, "Create &Filter...\tCtrl-F");
  file_menu->Append(wxID_OPEN, "&Open...\tCtrl-O");
  file_menu->Append(wxID_SAVE, "&Save\tCtrl-S");
  file_menu->Append(wxID_SAVEAS, "Save &As...");
  file_menu->Append(wxID_CLOSE, "&Close\tCtrl-W");
  
  file_menu->AppendSeparator();
  file_menu->Append(IFMENU_FILE_PROPERTIES, "P&roperties");
  file_menu->Append(IFMENU_FILE_EXPORT, "&Export...");
  
  file_menu->AppendSeparator();
  file_menu->Append(wxID_PRINT, "&Print...");
  file_menu->Append(wxID_PRINT_SETUP, "Print &Setup...");
  file_menu->Append(wxID_PREVIEW, "Print Pre&view");
  
  wxMenu *view_menu = new wxMenu;
  view_menu->Append(IFMENU_VIEW_SCALE_MINMAX, "Display Scale S&et...\tCtrl-E");
  view_menu->Append(IFMENU_VIEW_SCALE_AUTO, "Display Scale &Auto...\tCtrl-A");
  view_menu->Append(IFMENU_VIEW_SCALE_FULL, "Display F&ull Scale\tCtrl-U");
  
  wxMenu* filter_menu = new wxMenu;
  filter_menu->Append (IFMENU_FILTER_INVERTVALUES, "&Invert Values");
  filter_menu->Append (IFMENU_FILTER_SQUARE, "&Square");
  filter_menu->Append (IFMENU_FILTER_SQRT, "Square &Root");
  filter_menu->Append (IFMENU_FILTER_LOG, "&Log");
  filter_menu->Append (IFMENU_FILTER_EXP, "&Exp");
  filter_menu->AppendSeparator();
#ifdef HAVE_FFT
  filter_menu->Append (IFMENU_FILTER_FFT, "2D &FFT");
  filter_menu->Append (IFMENU_FILTER_IFFT, "2D &IFFT");
  filter_menu->Append (IFMENU_FILTER_FFT_ROWS, "FFT Rows");
  filter_menu->Append (IFMENU_FILTER_IFFT_ROWS, "IFFT Rows");
  filter_menu->Append (IFMENU_FILTER_FFT_COLS, "FFT Columns");
  filter_menu->Append (IFMENU_FILTER_IFFT_COLS, "IFFT Columns");
  filter_menu->Append (IFMENU_FILTER_FOURIER, "F&ourier");
  filter_menu->Append (IFMENU_FILTER_INVERSE_FOURIER, "Inverse Fo&urier");
#else
  filter_menu->Append (IFMENU_FILTER_FOURIER, "&Fourier");
  filter_menu->Append (IFMENU_FILTER_INVERSE_FOURIER, "&Inverse Fourier");
#endif
  filter_menu->Append (IFMENU_FILTER_SHUFFLEFOURIERTONATURALORDER, "S&huffle Fourier to Natural Order");
  filter_menu->Append (IFMENU_FILTER_SHUFFLENATURALTOFOURIERORDER, "Shu&ffle Natural to Fourier Order");
  filter_menu->Append (IFMENU_FILTER_MAGNITUDE, "&Magnitude");
  filter_menu->Append (IFMENU_FILTER_PHASE, "&Phase");
  
  wxMenu* image_menu = new wxMenu;
  image_menu->Append (IFMENU_IMAGE_ADD, "&Add...");
  image_menu->Append (IFMENU_IMAGE_SUBTRACT, "&Subtract...");
  image_menu->Append (IFMENU_IMAGE_MULTIPLY, "&Multiply...");
  image_menu->Append (IFMENU_IMAGE_DIVIDE, "&Divide...");
  image_menu->AppendSeparator();
  image_menu->Append (IFMENU_IMAGE_SCALESIZE, "S&cale Size...");

  wxMenu *analyze_menu = new wxMenu;
  analyze_menu->Append (IFMENU_PLOT_ROW, "Plot &Row");
  analyze_menu->Append (IFMENU_PLOT_COL, "Plot &Column");
  analyze_menu->Append (IFMENU_PLOT_HISTOGRAM, "Plot &Histogram");
  analyze_menu->AppendSeparator();
  analyze_menu->Append (IFMENU_PLOT_FFT_ROW, "Plot FFT Row");
  analyze_menu->Append (IFMENU_PLOT_FFT_COL, "Plot FFT Column");
  analyze_menu->AppendSeparator();
  analyze_menu->Append (IFMENU_COMPARE_IMAGES, "Compare &Images...");
  analyze_menu->Append (IFMENU_COMPARE_ROW, "Compare &Row");
  analyze_menu->Append (IFMENU_COMPARE_COL, "Compare &Column");
  
  wxMenu *help_menu = new wxMenu;
  help_menu->Append(MAINMENU_HELP_CONTENTS, "&Contents\tF1");
  help_menu->Append(MAINMENU_HELP_TOPICS, "&Topics\tCtrl-H");
  help_menu->Append(MAINMENU_HELP_ABOUT, "&About");
  
  wxMenuBar *menu_bar = new wxMenuBar;
  
  menu_bar->Append(file_menu, "&File");
  menu_bar->Append(view_menu, "&View");
  menu_bar->Append(image_menu, "&Image");
  menu_bar->Append(filter_menu, "Fi&lter");
  menu_bar->Append(analyze_menu, "&Analyze");
  menu_bar->Append(help_menu, "&Help");

  subframe->SetMenuBar(menu_bar);
  
  subframe->Centre(wxBOTH);
  
  wxAcceleratorEntry accelEntries[10];
  accelEntries[0].Set (wxACCEL_CTRL, static_cast<int>('O'), wxID_OPEN);
  accelEntries[1].Set (wxACCEL_CTRL, static_cast<int>('S'), wxID_SAVE);
  accelEntries[2].Set (wxACCEL_CTRL, static_cast<int>('W'), wxID_CLOSE);
  accelEntries[3].Set (wxACCEL_CTRL, static_cast<int>('H'), MAINMENU_HELP_TOPICS);
  accelEntries[4].Set (wxACCEL_CTRL, static_cast<int>('P'), MAINMENU_FILE_CREATE_PHANTOM);
  accelEntries[5].Set (wxACCEL_CTRL, static_cast<int>('F'), MAINMENU_FILE_CREATE_FILTER);
  accelEntries[6].Set (wxACCEL_NORMAL, WXK_F1, MAINMENU_HELP_CONTENTS);
  accelEntries[7].Set (wxACCEL_CTRL, static_cast<int>('A'), IFMENU_VIEW_SCALE_AUTO);
  accelEntries[8].Set (wxACCEL_CTRL, static_cast<int>('U'), IFMENU_VIEW_SCALE_FULL);
  accelEntries[9].Set (wxACCEL_CTRL, static_cast<int>('E'), IFMENU_VIEW_SCALE_MINMAX);
  wxAcceleratorTable accelTable (10, accelEntries);
  subframe->SetAcceleratorTable (accelTable);

  return subframe;
}


bool 
ImageFileView::OnCreate (wxDocument *doc, long WXUNUSED(flags) )
{
  m_frame = CreateChildFrame(doc, this);
  SetFrame (m_frame);
  
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  m_dAutoScaleFactor = 1.;
  
  int width, height;
  m_frame->GetClientSize (&width, &height);
  m_frame->SetTitle("ImageFileView");
  m_canvas = CreateCanvas (this, m_frame);
  
  int x, y;  // X requires a forced resize
  m_frame->GetSize(&x, &y);
  m_frame->SetSize(-1, -1, x, y);
  m_frame->SetFocus();
  m_frame->Show(true);
  Activate(true);
  
  return true;
}

void 
ImageFileView::OnDraw (wxDC* dc)
{
  wxSize sizeWindow = m_frame->GetClientSize();
  wxSize sizeBest = m_canvas->GetBestSize();
  if (sizeWindow.x > sizeBest.x || sizeWindow.y > sizeBest.y)
    m_frame->SetClientSize (sizeBest);

  if (m_bitmap.Ok())
    dc->DrawBitmap(m_bitmap, 0, 0, false);
  
  int xCursor, yCursor;
  if (m_canvas->GetCurrentCursor (xCursor, yCursor))
    m_canvas->DrawRubberBandCursor (*dc, xCursor, yCursor);
}


void 
ImageFileView::OnUpdate (wxView *WXUNUSED(sender), wxObject *WXUNUSED(hint) )
{
  const ImageFile& rIF = dynamic_cast<ImageFileDocument*>(GetDocument())->getImageFile();
  ImageFileArrayConst v = rIF.getArray();
  int nx = rIF.nx();
  int ny = rIF.ny();
  if (v != NULL && nx != 0 && ny != 0) {
    if (! m_bMinSpecified || ! m_bMaxSpecified) {
      double min, max;
      rIF.getMinMax (min, max);
      if (! m_bMinSpecified)
        m_dMinPixel = min;
      if (! m_bMaxSpecified)
        m_dMaxPixel = max;
    }
    double scaleWidth = m_dMaxPixel - m_dMinPixel;
    
    unsigned char* imageData = new unsigned char [nx * ny * 3];
    for (int ix = 0; ix < nx; ix++) {
      for (int iy = 0; iy < ny; iy++) {
        double scaleValue = ((v[ix][iy] - m_dMinPixel) / scaleWidth) * 255;
        int intensity = static_cast<int>(scaleValue + 0.5);
        intensity = clamp (intensity, 0, 255);
        int baseAddr = ((ny - 1 - iy) * nx + ix) * 3;
        imageData[baseAddr] = imageData[baseAddr+1] = imageData[baseAddr+2] = intensity;
      }
    }
    wxImage image (nx, ny, imageData, true);
    m_bitmap = image.ConvertToBitmap();
    delete imageData;
    int xSize = nx;
    int ySize = ny;
    xSize = clamp (xSize, 0, 800);
    ySize = clamp (ySize, 0, 800);
    m_frame->SetClientSize (xSize, ySize);
    m_canvas->SetScrollbars(20, 20, nx/20, ny/20);
    m_canvas->SetBackgroundColour(*wxWHITE);
  } 
  
  if (m_canvas)
    m_canvas->Refresh();
}

bool 
ImageFileView::OnClose (bool deleteWindow)
{
  if (!GetDocument()->Close())
    return false;
  
  m_canvas->Clear();
  //m_canvas->m_pView = NULL;
  m_canvas = NULL;
  wxString s(theApp->GetAppName());
  if (m_frame)
    m_frame->SetTitle(s);
  SetFrame(NULL);
  
  Activate(false);
  
  if (deleteWindow) {
    delete m_frame;
    return true;
  }
  return true;
}

void
ImageFileView::OnExport (wxCommandEvent& event)
{
  ImageFile& rIF = dynamic_cast<ImageFileDocument*>(GetDocument())->getImageFile();
  ImageFileArrayConst v = rIF.getArray();
  int nx = rIF.nx();
  int ny = rIF.ny();
  if (v != NULL && nx != 0 && ny != 0) {
    if (! m_bMinSpecified || ! m_bMaxSpecified) {
      double min, max;
      rIF.getMinMax (min, max);
      if (! m_bMinSpecified)
        m_dMinPixel = min;
      if (! m_bMaxSpecified)
        m_dMaxPixel = max;
    }

    DialogExportParameters dialogExport (getFrameForChild(), m_iDefaultExportFormatID);
    if (dialogExport.ShowModal() == wxID_OK) {
      wxString strFormatName (dialogExport.getFormatName ());
      m_iDefaultExportFormatID = ImageFile::convertFormatNameToID (strFormatName.c_str());

      wxString strExt;
      wxString strWildcard;
      if (m_iDefaultExportFormatID == ImageFile::FORMAT_PGM || m_iDefaultExportFormatID == ImageFile::FORMAT_PGMASCII) {
        strExt = ".pgm";
        strWildcard = "PGM Files (*.pgm)|*.pgm";
      }
#ifdef HAVE_PNG
      else if (m_iDefaultExportFormatID == ImageFile::FORMAT_PNG || m_iDefaultExportFormatID == ImageFile::FORMAT_PNG16) {
        strExt = ".png";
        strWildcard = "PNG Files (*.png)|*.png";
      }
#endif

      const wxString& strFilename = wxFileSelector (wxString("Export Filename"), wxString(""), 
        wxString(""), strExt, strWildcard, wxOVERWRITE_PROMPT | wxHIDE_READONLY | wxSAVE);
      if (strFilename) {
        rIF.exportImage (strFormatName.c_str(), strFilename.c_str(), 1, 1, m_dMinPixel, m_dMaxPixel);
        *theApp->getLog() << "Exported file " << strFilename << "\n";
      }
    }
  }
}

void
ImageFileView::OnScaleSize (wxCommandEvent& event)
{
  ImageFile& rIF = GetDocument()->getImageFile();
  unsigned int iOldNX = rIF.nx();
  unsigned int iOldNY = rIF.ny();

  DialogGetXYSize dialogGetXYSize (getFrameForChild(), "Set New X & Y Dimensions", iOldNX, iOldNY);
  if (dialogGetXYSize.ShowModal() == wxID_OK) {
    unsigned int iNewNX = dialogGetXYSize.getXSize();
    unsigned int iNewNY = dialogGetXYSize.getYSize();
    std::ostringstream os;
    os << "Scale Size from (" << iOldNX << "," << iOldNY << ") to (" << iNewNX << "," << iNewNY << ")";
    ImageFileDocument* pScaledDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
    if (! pScaledDoc) {
      sys_error (ERR_SEVERE, "Unable to create image file");
      return;
    }
    ImageFile& rScaledIF = pScaledDoc->getImageFile();
    rScaledIF.setArraySize (iNewNX, iNewNY);
    rScaledIF.labelsCopy (rIF);
    rScaledIF.labelAdd (os.str().c_str());
    rIF.scaleImage (rScaledIF);
    *theApp->getLog() << os.str().c_str() << "\n";
    if (theApp->getSetModifyNewDocs())
      pScaledDoc->Modify(TRUE);
    pScaledDoc->UpdateAllViews (this);
    pScaledDoc->GetFirstView()->OnUpdate (this, NULL);
  }
}

void
ImageFileView::OnPlotRow (wxCommandEvent& event)
{
  int xCursor, yCursor;
  if (! m_canvas->GetCurrentCursor (xCursor, yCursor)) {
    wxMessageBox ("No row selected. Please use left mouse button on image to select column","Error");
    return;
  }
  
  const ImageFile& rIF = dynamic_cast<ImageFileDocument*>(GetDocument())->getImageFile();
  ImageFileArrayConst v = rIF.getArray();
  ImageFileArrayConst vImag = rIF.getImaginaryArray();
  int nx = rIF.nx();
  int ny = rIF.ny();
  
  if (v != NULL && yCursor < ny) {
    double* pX = new double [nx];
    double* pYReal = new double [nx];
    double *pYImag = NULL;
    double *pYMag = NULL;
    if (rIF.isComplex()) {
      pYImag = new double [nx];
      pYMag = new double [nx];
    }
    for (int i = 0; i < nx; i++) {
      pX[i] = i;
      pYReal[i] = v[i][yCursor];
      if (rIF.isComplex()) {
        pYImag[i] = vImag[i][yCursor];
        pYMag[i] = ::sqrt (v[i][yCursor] * v[i][yCursor] + vImag[i][yCursor] * vImag[i][yCursor]);
      }
    }
    PlotFileDocument* pPlotDoc = dynamic_cast<PlotFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.plt", wxDOC_SILENT));
    if (! pPlotDoc) {
      sys_error (ERR_SEVERE, "Internal error: unable to create Plot file");
    } else {
      PlotFile& rPlotFile = pPlotDoc->getPlotFile();
      std::ostringstream os;
      os << "Row " << yCursor;
      std::string title("title ");
      title += os.str();
      rPlotFile.addEzsetCommand (title.c_str());
      rPlotFile.addEzsetCommand ("xlabel Column");
      rPlotFile.addEzsetCommand ("ylabel Pixel Value");
      rPlotFile.addEzsetCommand ("lxfrac 0");
      rPlotFile.addEzsetCommand ("box");
      rPlotFile.addEzsetCommand ("grid");
      rPlotFile.addEzsetCommand ("curve 1");
      rPlotFile.addEzsetCommand ("color 1");
      if (rIF.isComplex()) {
        rPlotFile.addEzsetCommand ("dash 1");
        rPlotFile.addEzsetCommand ("curve 2");
        rPlotFile.addEzsetCommand ("color 4");
        rPlotFile.addEzsetCommand ("dash 3");
        rPlotFile.addEzsetCommand ("curve 3");
        rPlotFile.addEzsetCommand ("color 0");
        rPlotFile.addEzsetCommand ("solid");
        rPlotFile.setCurveSize (4, nx);
      } else
        rPlotFile.setCurveSize (2, nx);
      rPlotFile.addColumn (0, pX);
      rPlotFile.addColumn (1, pYReal); 
      if (rIF.isComplex()) {
        rPlotFile.addColumn (2, pYImag);
        rPlotFile.addColumn (3, pYMag);
      }
      for (unsigned int iL = 0; iL < rIF.nLabels(); iL++)
        rPlotFile.addDescription (rIF.labelGet(iL).getLabelString().c_str());
      os << " Plot of " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str();
      *theApp->getLog() << os.str().c_str() << "\n";
      rPlotFile.addDescription (os.str().c_str());
    }
    delete pX;
    delete pYReal;
    if (rIF.isComplex()) {
      delete pYImag;
      delete pYMag;
    }
    if (theApp->getSetModifyNewDocs())
      pPlotDoc->Modify(true);
    pPlotDoc->UpdateAllViews();
  }
}

void
ImageFileView::OnPlotCol (wxCommandEvent& event)
{
  int xCursor, yCursor;
  if (! m_canvas->GetCurrentCursor (xCursor, yCursor)) {
    wxMessageBox ("No column selected. Please use left mouse button on image to select column","Error");
    return;
  }
  
  const ImageFile& rIF = dynamic_cast<ImageFileDocument*>(GetDocument())->getImageFile();
  ImageFileArrayConst v = rIF.getArray();
  ImageFileArrayConst vImag = rIF.getImaginaryArray();
  int nx = rIF.nx();
  int ny = rIF.ny();
  
  if (v != NULL && xCursor < nx) {
    double* pX = new double [ny];
    double* pYReal = new double [ny];
    double* pYImag = NULL;
    double* pYMag = NULL;
    if (rIF.isComplex()) {
      pYImag = new double [ny];
      pYMag = new double [ny];
    }
    for (int i = 0; i < ny; i++) {
      pX[i] = i;
      pYReal[i] = v[xCursor][i];
      if (rIF.isComplex()) {
        pYImag[i] = vImag[xCursor][i];
        pYMag[i] = ::sqrt (v[xCursor][i] * v[xCursor][i] + vImag[xCursor][i] * vImag[xCursor][i]);
      }
    }
    PlotFileDocument* pPlotDoc = dynamic_cast<PlotFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.plt", wxDOC_SILENT));
    if (! pPlotDoc) {
      sys_error (ERR_SEVERE, "Internal error: unable to create Plot file");
    } else {
      PlotFile& rPlotFile = pPlotDoc->getPlotFile();
      std::ostringstream os;
      os << "Column " << xCursor;
      std::string title("title ");
      title += os.str();
      rPlotFile.addEzsetCommand (title.c_str());
      rPlotFile.addEzsetCommand ("xlabel Row");
      rPlotFile.addEzsetCommand ("ylabel Pixel Value");
      rPlotFile.addEzsetCommand ("lxfrac 0");
      rPlotFile.addEzsetCommand ("box");
      rPlotFile.addEzsetCommand ("grid");
      rPlotFile.addEzsetCommand ("curve 1");
      rPlotFile.addEzsetCommand ("color 1");
      if (rIF.isComplex()) {
        rPlotFile.addEzsetCommand ("dash 1");
        rPlotFile.addEzsetCommand ("curve 2");
        rPlotFile.addEzsetCommand ("color 4");
        rPlotFile.addEzsetCommand ("dash 3");
        rPlotFile.addEzsetCommand ("curve 3");
        rPlotFile.addEzsetCommand ("color 0");
        rPlotFile.addEzsetCommand ("solid");
        rPlotFile.setCurveSize (4, ny);
      } else
        rPlotFile.setCurveSize (2, ny);
      rPlotFile.addColumn (0, pX);
      rPlotFile.addColumn (1, pYReal); 
      if (rIF.isComplex()) {
        rPlotFile.addColumn (2, pYImag);
        rPlotFile.addColumn (3, pYMag);
      }
      for (unsigned int iL = 0; iL < rIF.nLabels(); iL++)
        rPlotFile.addDescription (rIF.labelGet(iL).getLabelString().c_str());
      os << " Plot of " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str();
      *theApp->getLog() << os.str().c_str() << "\n";
      rPlotFile.addDescription (os.str().c_str());
    }
    delete pX;
    delete pYReal;
    if (rIF.isComplex()) {
      delete pYImag;
      delete pYMag;
    }
    if (theApp->getSetModifyNewDocs())
      pPlotDoc->Modify(true);
    pPlotDoc->UpdateAllViews();
  }
}

#ifdef HAVE_FFT
void
ImageFileView::OnPlotFFTRow (wxCommandEvent& event)
{
  int xCursor, yCursor;
  if (! m_canvas->GetCurrentCursor (xCursor, yCursor)) {
    wxMessageBox ("No row selected. Please use left mouse button on image to select column","Error");
    return;
  }
  
  const ImageFile& rIF = dynamic_cast<ImageFileDocument*>(GetDocument())->getImageFile();
  ImageFileArrayConst v = rIF.getArray();
  ImageFileArrayConst vImag = rIF.getImaginaryArray();
  int nx = rIF.nx();
  int ny = rIF.ny();
  
  if (v != NULL && yCursor < ny) {
    fftw_complex* pcIn = new fftw_complex [nx];

    int i;
    for (i = 0; i < nx; i++) {
      pcIn[i].re = v[i][yCursor];
      if (rIF.isComplex())
        pcIn[i].im = vImag[i][yCursor];
      else
        pcIn[i].im = 0;
    }

    fftw_plan plan = fftw_create_plan (nx, FFTW_FORWARD, FFTW_IN_PLACE);
    fftw_one (plan, pcIn, NULL);
    fftw_destroy_plan (plan);

    double* pX = new double [nx];
    double* pYReal = new double [nx];
    double* pYImag = new double [nx];
    double* pYMag = new double [nx];
    for (i = 0; i < nx; i++) {
      pX[i] = i;
      pYReal[i] = pcIn[i].re;
      pYImag[i] = pcIn[i].im;
      pYMag[i] = ::sqrt (pcIn[i].re * pcIn[i].re + pcIn[i].im * pcIn[i].im);
    }
    Fourier::shuffleFourierToNaturalOrder (pYReal, nx);
    Fourier::shuffleFourierToNaturalOrder (pYImag, nx);
    Fourier::shuffleFourierToNaturalOrder (pYMag, nx);

    PlotFileDocument* pPlotDoc = dynamic_cast<PlotFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.plt", wxDOC_SILENT));
    if (! pPlotDoc) {
      sys_error (ERR_SEVERE, "Internal error: unable to create Plot file");
    } else {
      PlotFile& rPlotFile = pPlotDoc->getPlotFile();
      std::ostringstream os;
      os << "Row " << yCursor;
      std::string title("title ");
      title += os.str();
      rPlotFile.addEzsetCommand (title.c_str());
      rPlotFile.addEzsetCommand ("xlabel Column");
      rPlotFile.addEzsetCommand ("ylabel Pixel Value");
      rPlotFile.addEzsetCommand ("lxfrac 0");
      rPlotFile.addEzsetCommand ("curve 1");
      rPlotFile.addEzsetCommand ("color 1");
       rPlotFile.addEzsetCommand ("dash 1");
        rPlotFile.addEzsetCommand ("curve 2");
        rPlotFile.addEzsetCommand ("color 4");
        rPlotFile.addEzsetCommand ("dash 3");
        rPlotFile.addEzsetCommand ("curve 3");
        rPlotFile.addEzsetCommand ("color 0");
        rPlotFile.addEzsetCommand ("solid");
       rPlotFile.addEzsetCommand ("box");
      rPlotFile.addEzsetCommand ("grid");
      rPlotFile.setCurveSize (4, nx);
      rPlotFile.addColumn (0, pX);
      rPlotFile.addColumn (1, pYReal);
      rPlotFile.addColumn (2, pYImag);
      rPlotFile.addColumn (3, pYMag);
      for (int iL = 0; iL < rIF.nLabels(); iL++)
        rPlotFile.addDescription (rIF.labelGet(iL).getLabelString().c_str());
      os << " FFT Plot of " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str();
      *theApp->getLog() << os.str().c_str() << "\n";
      rPlotFile.addDescription (os.str().c_str());
    }
    delete pX;
    delete pYReal;
    delete pYImag;
    delete pYMag;
    delete [] pcIn;

    if (theApp->getSetModifyNewDocs())
      pPlotDoc->Modify(true);
    pPlotDoc->UpdateAllViews();
  }
}

void
ImageFileView::OnPlotFFTCol (wxCommandEvent& event)
{
  int xCursor, yCursor;
  if (! m_canvas->GetCurrentCursor (xCursor, yCursor)) {
    wxMessageBox ("No column selected. Please use left mouse button on image to select column","Error");
    return;
  }
  
  const ImageFile& rIF = dynamic_cast<ImageFileDocument*>(GetDocument())->getImageFile();
  ImageFileArrayConst v = rIF.getArray();
  ImageFileArrayConst vImag = rIF.getImaginaryArray();
  int nx = rIF.nx();
  int ny = rIF.ny();
  
  if (v != NULL && xCursor < nx) {
    fftw_complex* pcIn = new fftw_complex [ny];
    double *pdTemp = new double [ny];

    int i;
    for (i = 0; i < ny; i++)
      pdTemp[i] = v[xCursor][i];
    Fourier::shuffleNaturalToFourierOrder (pdTemp, ny);
    for (i = 0; i < ny; i++) 
      pcIn[i].re = pdTemp[i];

    for (i = 0; i < ny; i++) {
      if (rIF.isComplex())
        pdTemp[i] = vImag[xCursor][i];
      else
      pdTemp[i] = 0;
    }
    Fourier::shuffleNaturalToFourierOrder (pdTemp, ny);
    for (i = 0; i < ny; i++)
      pcIn[i].im = pdTemp[i];

    fftw_plan plan = fftw_create_plan (ny, FFTW_BACKWARD, FFTW_IN_PLACE);
    fftw_one (plan, pcIn, NULL);
    fftw_destroy_plan (plan);

    double* pX = new double [ny];
    double* pYReal = new double [ny];
    double* pYImag = new double [ny];
    double* pYMag = new double [ny];
    for (i = 0; i < ny; i++) {
      pX[i] = i;
      pYReal[i] = pcIn[i].re;
      pYImag[i] = pcIn[i].im;
      pYMag[i] = ::sqrt (pcIn[i].re * pcIn[i].re + pcIn[i].im * pcIn[i].im);
    }

    PlotFileDocument* pPlotDoc = dynamic_cast<PlotFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.plt", wxDOC_SILENT));
    if (! pPlotDoc) {
      sys_error (ERR_SEVERE, "Internal error: unable to create Plot file");
    } else {
      PlotFile& rPlotFile = pPlotDoc->getPlotFile();
      std::ostringstream os;
      os << "Column " << xCursor;
      std::string title("title ");
      title += os.str();
      rPlotFile.addEzsetCommand (title.c_str());
      rPlotFile.addEzsetCommand ("xlabel Column");
      rPlotFile.addEzsetCommand ("ylabel Pixel Value");
      rPlotFile.addEzsetCommand ("lxfrac 0");
      rPlotFile.addEzsetCommand ("curve 1");
      rPlotFile.addEzsetCommand ("color 1");
       rPlotFile.addEzsetCommand ("dash 1");
        rPlotFile.addEzsetCommand ("curve 2");
        rPlotFile.addEzsetCommand ("color 4");
        rPlotFile.addEzsetCommand ("dash 3");
        rPlotFile.addEzsetCommand ("curve 3");
        rPlotFile.addEzsetCommand ("color 0");
        rPlotFile.addEzsetCommand ("solid");
       rPlotFile.addEzsetCommand ("box");
      rPlotFile.addEzsetCommand ("grid");
      rPlotFile.setCurveSize (4, ny);
      rPlotFile.addColumn (0, pX);
      rPlotFile.addColumn (1, pYReal);
      rPlotFile.addColumn (2, pYImag);
      rPlotFile.addColumn (3, pYMag);
      for (int iL = 0; iL < rIF.nLabels(); iL++)
        rPlotFile.addDescription (rIF.labelGet(iL).getLabelString().c_str());
      os << " FFT Plot of " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str();
      *theApp->getLog() << os.str().c_str() << "\n";
      rPlotFile.addDescription (os.str().c_str());
    }
    delete pX;
    delete pYReal;
    delete pYImag;
    delete pYMag;
    delete pdTemp;
    delete [] pcIn;

    if (theApp->getSetModifyNewDocs())
      pPlotDoc->Modify(true);
    pPlotDoc->UpdateAllViews();
  }
}
#endif

void
ImageFileView::OnCompareCol (wxCommandEvent& event)
{
  int xCursor, yCursor;
  if (! m_canvas->GetCurrentCursor (xCursor, yCursor)) {
    wxMessageBox ("No column selected. Please use left mouse button on image to select column","Error");
    return;
  }
  
  std::vector<ImageFileDocument*> vecIFDoc;
  theApp->getCompatibleImages (GetDocument(), vecIFDoc);
  if (vecIFDoc.size() == 0) {
    wxMessageBox ("No compatible images for Column Comparison", "Error");
    return;
  }
  DialogGetComparisonImage dialogGetCompare (getFrameForChild(), "Get Comparison Image", vecIFDoc, false);
  
  if (dialogGetCompare.ShowModal() == wxID_OK) {
    ImageFileDocument* pCompareDoc = dialogGetCompare.getImageFileDocument();
    const ImageFile& rIF = GetDocument()->getImageFile();
    const ImageFile& rCompareIF = pCompareDoc->getImageFile();
    
    ImageFileArrayConst v1 = rIF.getArray();
    ImageFileArrayConst v2 = rCompareIF.getArray();
    int nx = rIF.nx();
    int ny = rIF.ny();
    
    if (v1 != NULL && xCursor < nx) {
      double* pX = new double [ny];
      double* pY1 = new double [ny];
      double* pY2 = new double [ny];
      for (int i = 0; i < ny; i++) {
        pX[i] = i;
        pY1[i] = v1[xCursor][i];
        pY2[i] = v2[xCursor][i];
      }
      PlotFileDocument* pPlotDoc = dynamic_cast<PlotFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.plt", wxDOC_SILENT));
      if (! pPlotDoc) {
        sys_error (ERR_SEVERE, "Internal error: unable to create Plot file");
      } else {
        PlotFile& rPlotFile = pPlotDoc->getPlotFile();
        std::ostringstream os;
        os << "Column " << xCursor << " Comparison";
        std::string title("title ");
        title += os.str();
        rPlotFile.addEzsetCommand (title.c_str());
        rPlotFile.addEzsetCommand ("xlabel Row");
        rPlotFile.addEzsetCommand ("ylabel Pixel Value");
        rPlotFile.addEzsetCommand ("lxfrac 0");
        rPlotFile.addEzsetCommand ("curve 1");
        rPlotFile.addEzsetCommand ("color 2");
        rPlotFile.addEzsetCommand ("curve 2");
        rPlotFile.addEzsetCommand ("color 4");
        rPlotFile.addEzsetCommand ("dash 5");
        rPlotFile.addEzsetCommand ("box");
        rPlotFile.addEzsetCommand ("grid");
        rPlotFile.setCurveSize (3, ny);
        rPlotFile.addColumn (0, pX);
        rPlotFile.addColumn (1, pY1);
        rPlotFile.addColumn (2, pY2);

        unsigned int iL;
        for (iL = 0; iL < rIF.nLabels(); iL++) {
          std::string s = GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str();
          s += ": ";
          s += rIF.labelGet(iL).getLabelString();
          rPlotFile.addDescription (s.c_str());
        }
        for (iL = 0; iL < rCompareIF.nLabels(); iL++) {
          std::string s = pCompareDoc->GetFirstView()->GetFrame()->GetTitle().c_str();
          s += ": ";
          s += rCompareIF.labelGet(iL).getLabelString();
          rPlotFile.addDescription (s.c_str());
        }
        os << " Between " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str() << " and "
          << pCompareDoc->GetFirstView()->GetFrame()->GetTitle().c_str();
        *theApp->getLog() << os.str().c_str() << "\n";
        rPlotFile.addDescription (os.str().c_str());
      }
      delete pX;
      delete pY1;
      delete pY2;
      if (theApp->getSetModifyNewDocs())
        pPlotDoc->Modify(true);
      pPlotDoc->UpdateAllViews();
    }
  }
}

void
ImageFileView::OnCompareRow (wxCommandEvent& event)
{
  int xCursor, yCursor;
  if (! m_canvas->GetCurrentCursor (xCursor, yCursor)) {
    wxMessageBox ("No column selected. Please use left mouse button on image to select column","Error");
    return;
  }
  
  std::vector<ImageFileDocument*> vecIFDoc;
  theApp->getCompatibleImages (GetDocument(), vecIFDoc);
  
  if (vecIFDoc.size() == 0) {
    wxMessageBox ("No compatible images for Row Comparison", "Error");
    return;
  }
  
  DialogGetComparisonImage dialogGetCompare (getFrameForChild(), "Get Comparison Image", vecIFDoc, false);
  
  if (dialogGetCompare.ShowModal() == wxID_OK) {
    ImageFileDocument* pCompareDoc = dialogGetCompare.getImageFileDocument();
    const ImageFile& rIF = GetDocument()->getImageFile();
    const ImageFile& rCompareIF = pCompareDoc->getImageFile();
    
    ImageFileArrayConst v1 = rIF.getArray();
    ImageFileArrayConst v2 = rCompareIF.getArray();
    int nx = rIF.nx();
    int ny = rIF.ny();
    
    if (v1 != NULL && yCursor < ny) {
      double* pX = new double [nx];
      double* pY1 = new double [nx];
      double* pY2 = new double [nx];
      for (int i = 0; i < nx; i++) {
        pX[i] = i;
        pY1[i] = v1[i][yCursor];
        pY2[i] = v2[i][yCursor];
      }
      PlotFileDocument* pPlotDoc = dynamic_cast<PlotFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.plt", wxDOC_SILENT));
      if (! pPlotDoc) {
        sys_error (ERR_SEVERE, "Internal error: unable to create Plot file");
      } else {
        PlotFile& rPlotFile = pPlotDoc->getPlotFile();
        std::ostringstream os;
        os << "Row " << yCursor << " Comparison";
        std::string title("title ");
        title += os.str();
        rPlotFile.addEzsetCommand (title.c_str());
        rPlotFile.addEzsetCommand ("xlabel Column");
        rPlotFile.addEzsetCommand ("ylabel Pixel Value");
        rPlotFile.addEzsetCommand ("lxfrac 0");
        rPlotFile.addEzsetCommand ("curve 1");
        rPlotFile.addEzsetCommand ("color 2");
        rPlotFile.addEzsetCommand ("curve 2");
        rPlotFile.addEzsetCommand ("color 4");
        rPlotFile.addEzsetCommand ("dash 5");
        rPlotFile.addEzsetCommand ("box");
        rPlotFile.addEzsetCommand ("grid");
        rPlotFile.setCurveSize (3, nx);
        rPlotFile.addColumn (0, pX);
        rPlotFile.addColumn (1, pY1);
        rPlotFile.addColumn (2, pY2);
        unsigned int iL;
        for (iL = 0; iL < rIF.nLabels(); iL++) {
          std::string s = GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str();
          s += ": ";
          s += rIF.labelGet(iL).getLabelString();
          rPlotFile.addDescription (s.c_str());
        }
        for (iL = 0; iL < rCompareIF.nLabels(); iL++) {
          std::string s = pCompareDoc->GetFirstView()->GetFrame()->GetTitle().c_str();
          s += ": ";
          s += rCompareIF.labelGet(iL).getLabelString();
          rPlotFile.addDescription (s.c_str());
        }
        os << " Between " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str() << " and "
          << pCompareDoc->GetFirstView()->GetFrame()->GetTitle().c_str();
        *theApp->getLog() << os.str().c_str() << "\n";
        rPlotFile.addDescription (os.str().c_str());
      }
      delete pX;
      delete pY1;
      delete pY2;
      if (theApp->getSetModifyNewDocs())
        pPlotDoc->Modify(true);
      pPlotDoc->UpdateAllViews();
    }
  }
}

static int NUMBER_HISTOGRAM_BINS = 256;

void
ImageFileView::OnPlotHistogram (wxCommandEvent& event)
{ 
  const ImageFile& rIF = dynamic_cast<ImageFileDocument*>(GetDocument())->getImageFile();
  ImageFileArrayConst v = rIF.getArray();
  int nx = rIF.nx();
  int ny = rIF.ny();
  
  if (v != NULL && nx > 0 && ny > 0) {
    PlotFileDocument* pPlotDoc = dynamic_cast<PlotFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.plt", wxDOC_SILENT));
    if (! pPlotDoc) {
      sys_error (ERR_SEVERE, "Internal error: unable to create Plot file");
      return;
    }

    double* pX = new double [NUMBER_HISTOGRAM_BINS];
    double* pY = new double [NUMBER_HISTOGRAM_BINS];
    double dMin, dMax;
    rIF.getMinMax (dMin, dMax);
    double dBinWidth = (dMax - dMin) / NUMBER_HISTOGRAM_BINS;

    for (int i = 0; i < NUMBER_HISTOGRAM_BINS; i++) {
      pX[i] = dMin + (i + 0.5) * dBinWidth;
      pY[i] = 0;
    }
    for (int ix = 0; ix < nx; ix++)
      for (int iy = 0; iy < ny; iy++) {
        int iBin = nearest<int> ((v[ix][iy] - dMin) / dBinWidth);
        if (iBin >= 0 && iBin < NUMBER_HISTOGRAM_BINS)
          pY[iBin] += 1;
      }

    PlotFile& rPlotFile = pPlotDoc->getPlotFile();
    std::ostringstream os;
    os << "Histogram";
    std::string title("title ");
    title += os.str();
    rPlotFile.addEzsetCommand (title.c_str());
    rPlotFile.addEzsetCommand ("xlabel Pixel Value");
    rPlotFile.addEzsetCommand ("ylabel Count");
    rPlotFile.addEzsetCommand ("box");
    rPlotFile.addEzsetCommand ("grid");
    rPlotFile.setCurveSize (2, NUMBER_HISTOGRAM_BINS);
    rPlotFile.addColumn (0, pX);
    rPlotFile.addColumn (1, pY);
    for (unsigned int iL = 0; iL < rIF.nLabels(); iL++) {
      std::string s = GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str();
      s += ": ";
      s += rIF.labelGet(iL).getLabelString();
      rPlotFile.addDescription (s.c_str());
    }
    os << " Plot of " << GetDocument()->GetFirstView()->GetFrame()->GetTitle().c_str();
    *theApp->getLog() << os.str().c_str() << "\n";
    rPlotFile.addDescription (os.str().c_str());
    delete pX;
    delete pY;
    if (theApp->getSetModifyNewDocs())
      pPlotDoc->Modify(true);
    pPlotDoc->UpdateAllViews();
  }
}


// PhantomCanvas

PhantomCanvas::PhantomCanvas (PhantomView* v, wxFrame *frame, const wxPoint& pos, const wxSize& size, const long style)
: wxScrolledWindow(frame, -1, pos, size, style)
{
  m_pView = v;
}

void 
PhantomCanvas::OnDraw (wxDC& dc)
{
  if (m_pView)
    m_pView->OnDraw(& dc);
}


// PhantomView

IMPLEMENT_DYNAMIC_CLASS(PhantomView, wxView)

BEGIN_EVENT_TABLE(PhantomView, wxView)
EVT_MENU(PHMMENU_FILE_PROPERTIES, PhantomView::OnProperties)
EVT_MENU(PHMMENU_PROCESS_RASTERIZE, PhantomView::OnRasterize)
EVT_MENU(PHMMENU_PROCESS_PROJECTIONS, PhantomView::OnProjections)
END_EVENT_TABLE()

PhantomView::PhantomView(void) 
: wxView(), m_canvas(NULL), m_frame(NULL)
{
  m_iDefaultNDet = 367;
  m_iDefaultNView = 320;
  m_iDefaultNSample = 2;
  m_dDefaultRotation = 1;
  m_dDefaultFocalLength = 2;
  m_dDefaultFieldOfView = 1;
  m_iDefaultGeometry = Scanner::GEOMETRY_PARALLEL;
  m_iDefaultTrace = Trace::TRACE_NONE;

  m_iDefaultRasterNX = 256;
  m_iDefaultRasterNY = 256;
  m_iDefaultRasterNSamples = 2;
}

PhantomView::~PhantomView(void)
{
}

void
PhantomView::OnProperties (wxCommandEvent& event)
{
  const int idPhantom = GetDocument()->getPhantomID();
  const wxString& namePhantom = GetDocument()->getPhantomName();
  std::ostringstream os;
  os << "Phantom " << namePhantom.c_str() << " (" << idPhantom << ")" << "\n";
  const Phantom& rPhantom = GetDocument()->getPhantom();
  rPhantom.printDefinitions (os);
#if DEBUG
  rPhantom.print (os);
#endif
  *theApp->getLog() << os.str().c_str() << "\n";
  wxMessageBox (os.str().c_str(), "Phantom Properties");
}


void
PhantomView::OnProjections (wxCommandEvent& event)
{
  DialogGetProjectionParameters dialogProjection (getFrameForChild(), m_iDefaultNDet, m_iDefaultNView, m_iDefaultNSample, m_dDefaultRotation, m_dDefaultFocalLength, m_dDefaultFieldOfView, m_iDefaultGeometry, m_iDefaultTrace);
  int retVal = dialogProjection.ShowModal();
  if (retVal == wxID_OK) {
    m_iDefaultNDet = dialogProjection.getNDet();
    m_iDefaultNView = dialogProjection.getNView();
    m_iDefaultNSample = dialogProjection.getNSamples();
    m_iDefaultTrace = dialogProjection.getTrace();
    m_dDefaultRotation = dialogProjection.getRotAngle();
    m_dDefaultFocalLength = dialogProjection.getFocalLengthRatio();
    m_dDefaultFieldOfView = dialogProjection.getFieldOfViewRatio();
    wxString sGeometry = dialogProjection.getGeometry();
    m_iDefaultGeometry = Scanner::convertGeometryNameToID (sGeometry.c_str());
    
    if (m_iDefaultNDet > 0 && m_iDefaultNView > 0 && sGeometry != "") {
      const Phantom& rPhantom = GetDocument()->getPhantom();
      ProjectionFileDocument* pProjectionDoc = dynamic_cast<ProjectionFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.pj", wxDOC_SILENT));
      if (! pProjectionDoc) {
        sys_error (ERR_SEVERE, "Unable to create projection document");
        return;
      }
      Projections& rProj = pProjectionDoc->getProjections();
      Scanner theScanner (rPhantom, sGeometry.c_str(), m_iDefaultNDet, m_iDefaultNView, m_iDefaultNSample, m_dDefaultRotation, m_dDefaultFocalLength, m_dDefaultFieldOfView);
      if (theScanner.fail()) {
        *theApp->getLog() << "Failed making scanner: " << theScanner.failMessage().c_str() << "\n";
        return;
      }
      rProj.initFromScanner (theScanner);
      m_dDefaultRotation /= PI;  // convert back to PI units
      
      Timer timer;
      if (m_iDefaultTrace > Trace::TRACE_CONSOLE) {
        ProjectionsDialog dialogProjections (theScanner, rProj, rPhantom, m_iDefaultTrace, dynamic_cast<wxWindow*>(getFrameForChild()));
        for (int iView = 0; iView < rProj.nView(); iView++) {
          ::wxYield();
          ::wxYield();
          if (dialogProjections.isCancelled() || ! dialogProjections.projectView (iView)) {
            pProjectionDoc->DeleteAllViews();
            return;
          }
          ::wxYield();
          ::wxYield();
          while (dialogProjections.isPaused()) {
            ::wxYield();
            ::wxUsleep(50);
          }
        }
      } else {
        wxProgressDialog dlgProgress (wxString("Projection"), wxString("Projection Progress"), rProj.nView() + 1, getFrameForChild(), wxPD_CAN_ABORT);
        for (int i = 0; i < rProj.nView(); i++) {
          theScanner.collectProjections (rProj, rPhantom, i, 1, true, m_iDefaultTrace);
          if (! dlgProgress.Update (i+1)) {
            pProjectionDoc->DeleteAllViews();
            return;
          }
        }
      }
      
      std::ostringstream os;
      os << "Projections for " << rPhantom.name() << ": nDet=" << m_iDefaultNDet << ", nView=" << m_iDefaultNView << ", nSamples=" << m_iDefaultNSample << ", RotAngle=" << m_dDefaultRotation << ", FocalLengthRatio=" << m_dDefaultFocalLength << ", FieldOfViewRatio=" << m_dDefaultFieldOfView << ", Geometry=" << sGeometry.c_str();
      rProj.setCalcTime (timer.timerEnd());
      rProj.setRemark (os.str());
      *theApp->getLog() << os.str().c_str() << "\n";
      
      m_frame->Lower();
      ::wxYield();
      ProjectionFileView* projView = dynamic_cast<ProjectionFileView*>(pProjectionDoc->GetFirstView());
      if (projView) {
        projView->getFrame()->SetFocus();
        projView->OnUpdate (projView, NULL);
      }
      if (wxView* pView = pProjectionDoc->GetFirstView()) {
        if (wxFrame* pFrame = pView->GetFrame()) {
          pFrame->SetFocus();
          pFrame->Raise();
        }
        theApp->getDocManager()->ActivateView (pView, true, false);
      }
      ::wxYield();
      if (theApp->getSetModifyNewDocs())
        pProjectionDoc->Modify(true);
      pProjectionDoc->UpdateAllViews(this);
    }
  }
}


void
PhantomView::OnRasterize (wxCommandEvent& event)
{
  DialogGetRasterParameters dialogRaster (getFrameForChild(), m_iDefaultRasterNX, m_iDefaultRasterNY, m_iDefaultRasterNSamples);
  int retVal = dialogRaster.ShowModal();
  if (retVal == wxID_OK) {
    m_iDefaultRasterNX = dialogRaster.getXSize();
    m_iDefaultRasterNY  = dialogRaster.getYSize();
    m_iDefaultRasterNSamples = dialogRaster.getNSamples();
    if (m_iDefaultRasterNSamples < 1)
      m_iDefaultRasterNSamples = 1;
    if (m_iDefaultRasterNX > 0 && m_iDefaultRasterNY > 0) {
      const Phantom& rPhantom = GetDocument()->getPhantom();
      ImageFileDocument* pRasterDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
      if (! pRasterDoc) {
        sys_error (ERR_SEVERE, "Unable to create image file");
        return;
      }
      ImageFile& imageFile = pRasterDoc->getImageFile();
      
      imageFile.setArraySize (m_iDefaultRasterNX, m_iDefaultRasterNX);
      wxProgressDialog dlgProgress (wxString("Rasterize"), wxString("Rasterization Progress"), imageFile.nx() + 1, getFrameForChild(), wxPD_CAN_ABORT);
      Timer timer;
      for (unsigned int i = 0; i < imageFile.nx(); i++) {
        rPhantom.convertToImagefile (imageFile, m_iDefaultRasterNSamples, Trace::TRACE_NONE, i, 1, true);
        if (! dlgProgress.Update(i+1)) {
          pRasterDoc->DeleteAllViews();
          return;
        }
      }
      if (theApp->getSetModifyNewDocs())
        pRasterDoc->Modify(true);
      pRasterDoc->UpdateAllViews(this);
      std::ostringstream os;
      os << "Rasterize Phantom " << rPhantom.name() << ": XSize=" << m_iDefaultRasterNX << ", YSize=" 
        << m_iDefaultRasterNY << ", nSamples=" << m_iDefaultRasterNSamples;
      *theApp->getLog() << os.str().c_str() << "\n";
      imageFile.labelAdd (os.str().c_str(), timer.timerEnd());
      ImageFileView* rasterView = dynamic_cast<ImageFileView*>(pRasterDoc->GetFirstView());
      if (rasterView) {
        rasterView->getFrame()->SetFocus();
        rasterView->OnUpdate (rasterView, NULL);
      }
      
    }
  }
}


PhantomCanvas* 
PhantomView::CreateCanvas (wxView *view, wxFrame *parent)
{
  PhantomCanvas* pCanvas;
  int width, height;
  parent->GetClientSize(&width, &height);
  
  pCanvas = new PhantomCanvas (dynamic_cast<PhantomView*>(view), parent, wxPoint(0, 0), wxSize(width, height), 0);
  
  pCanvas->SetBackgroundColour(*wxWHITE);
  pCanvas->Clear();
  
  return pCanvas;
}

wxFrame*
PhantomView::CreateChildFrame(wxDocument *doc, wxView *view)
{
#if CTSIM_MDI
  wxDocMDIChildFrame *subframe = new wxDocMDIChildFrame (doc, view, theApp->getMainFrame(), -1, "Phantom Frame", wxPoint(10, 10), wxSize(256, 256), wxDEFAULT_FRAME_STYLE);
#else
  wxDocChildFrame *subframe = new wxDocChildFrame (doc, view, theApp->getMainFrame(), -1, "Phantom Frame", wxPoint(10, 10), wxSize(256, 256), wxDEFAULT_FRAME_STYLE);
#endif
  theApp->setIconForFrame (subframe);

  wxMenu *file_menu = new wxMenu;
  
  file_menu->Append(MAINMENU_FILE_CREATE_PHANTOM, "Cr&eate Phantom...\tCtrl-P");
  file_menu->Append(MAINMENU_FILE_CREATE_FILTER, "Create &Filter...\tCtrl-F");
  file_menu->Append(wxID_OPEN, "&Open...\tCtrl-O");
  file_menu->Append(wxID_SAVEAS, "Save &As...");
  file_menu->Append(wxID_CLOSE, "&Close");
  
  file_menu->AppendSeparator();
  file_menu->Append(PHMMENU_FILE_PROPERTIES, "P&roperties");
  
  file_menu->AppendSeparator();
  file_menu->Append(wxID_PRINT, "&Print...");
  file_menu->Append(wxID_PRINT_SETUP, "Print &Setup...");
  file_menu->Append(wxID_PREVIEW, "Print Pre&view");
  
  wxMenu *process_menu = new wxMenu;
  process_menu->Append(PHMMENU_PROCESS_RASTERIZE, "&Rasterize...\tCtrl-R");
  process_menu->Append(PHMMENU_PROCESS_PROJECTIONS, "&Projections...\tCtrl-J");
  
  wxMenu *help_menu = new wxMenu;
  help_menu->Append(MAINMENU_HELP_CONTENTS, "&Contents\tF1");
  help_menu->Append(MAINMENU_HELP_TOPICS, "&Topics\tCtrl-H");
  help_menu->Append(MAINMENU_HELP_ABOUT, "&About");
  
  wxMenuBar *menu_bar = new wxMenuBar;
  
  menu_bar->Append(file_menu, "&File");
  menu_bar->Append(process_menu, "&Process");
  menu_bar->Append(help_menu, "&Help");
  
  subframe->SetMenuBar(menu_bar);
  subframe->Centre(wxBOTH);

  wxAcceleratorEntry accelEntries[8];
  accelEntries[0].Set (wxACCEL_CTRL, static_cast<int>('O'), wxID_OPEN);
  accelEntries[1].Set (wxACCEL_CTRL, static_cast<int>('S'), wxID_SAVE);
  accelEntries[2].Set (wxACCEL_CTRL, static_cast<int>('H'), MAINMENU_HELP_TOPICS);
  accelEntries[3].Set (wxACCEL_CTRL, static_cast<int>('P'), MAINMENU_FILE_CREATE_PHANTOM);
  accelEntries[4].Set (wxACCEL_CTRL, static_cast<int>('F'), MAINMENU_FILE_CREATE_FILTER);
  accelEntries[5].Set (wxACCEL_NORMAL, WXK_F1, MAINMENU_HELP_CONTENTS);
  accelEntries[6].Set (wxACCEL_CTRL, static_cast<int>('J'), PHMMENU_PROCESS_PROJECTIONS);
  accelEntries[7].Set (wxACCEL_CTRL, static_cast<int>('R'), PHMMENU_PROCESS_RASTERIZE);
  wxAcceleratorTable accelTable (8, accelEntries);
  subframe->SetAcceleratorTable (accelTable);

  return subframe;
}


bool 
PhantomView::OnCreate(wxDocument *doc, long WXUNUSED(flags) )
{
  m_frame = CreateChildFrame(doc, this);
  SetFrame(m_frame);
  
  int width, height;
  m_frame->GetClientSize(&width, &height);
  m_frame->SetTitle("PhantomView");
  m_canvas = CreateCanvas (this, m_frame);
  
#ifdef __X__
  int x, y;  // X requires a forced resize
  m_frame->GetSize(&x, &y);
  m_frame->SetSize(-1, -1, x, y);
#endif
  
  m_frame->Show(true);
  Activate(true);
  
  return true;
}

void 
PhantomView::OnUpdate(wxView *WXUNUSED(sender), wxObject *WXUNUSED(hint) )
{
  if (m_canvas)
    m_canvas->Refresh();
}

bool 
PhantomView::OnClose (bool deleteWindow)
{
  if (!GetDocument()->Close())
    return false;
  
  m_canvas->Clear();
  //m_canvas->m_pView = NULL;
  m_canvas = NULL;
  wxString s(wxTheApp->GetAppName());
  if (m_frame)
    m_frame->SetTitle(s);
  SetFrame(NULL);
  
  Activate(false);
  
  if (deleteWindow)
    delete m_frame;

  return true;
}

void
PhantomView::OnDraw (wxDC* dc)
{
  int xsize, ysize;
  m_canvas->GetClientSize (&xsize, &ysize);
  SGPDriver driver (dc, xsize, ysize);
  SGP sgp (driver);
  const Phantom& rPhantom = GetDocument()->getPhantom();
  sgp.setColor (C_RED);
  rPhantom.show (sgp);
}

// ProjectionCanvas

ProjectionFileCanvas::ProjectionFileCanvas (ProjectionFileView* v, wxFrame *frame, const wxPoint& pos, const wxSize& size, const long style)
: wxScrolledWindow(frame, -1, pos, size, style)
{
  m_pView = v;
}

void 
ProjectionFileCanvas::OnDraw(wxDC& dc)
{
  if (m_pView)
    m_pView->OnDraw(& dc);
}

wxSize
ProjectionFileCanvas::GetBestSize () const
{
  wxSize best (0, 0);
  if (m_pView) {
    Projections& rProj = m_pView->GetDocument()->getProjections();
    best.Set (rProj.nDet(), rProj.nView());
  }

  return best;
}


// ProjectionFileView

IMPLEMENT_DYNAMIC_CLASS(ProjectionFileView, wxView)

BEGIN_EVENT_TABLE(ProjectionFileView, wxView)
EVT_MENU(PJMENU_FILE_PROPERTIES, ProjectionFileView::OnProperties)
EVT_MENU(PJMENU_RECONSTRUCT_FBP, ProjectionFileView::OnReconstructFBP)
EVT_MENU(PJMENU_CONVERT_POLAR, ProjectionFileView::OnConvertPolar)
EVT_MENU(PJMENU_CONVERT_FFT_POLAR, ProjectionFileView::OnConvertFFTPolar)
END_EVENT_TABLE()

ProjectionFileView::ProjectionFileView(void) 
: wxView(), m_canvas(NULL), m_frame(NULL)
{
  m_iDefaultNX = 256;
  m_iDefaultNY = 256;
  m_iDefaultFilter = SignalFilter::FILTER_ABS_BANDLIMIT;
  m_dDefaultFilterParam = 1.;
#if HAVE_FFTW
  m_iDefaultFilterMethod = ProcessSignal::FILTER_METHOD_RFFTW;
  m_iDefaultFilterGeneration = ProcessSignal::FILTER_GENERATION_INVERSE_FOURIER;
#else
  m_iDefaultFilterMethod = ProcessSignal::FILTER_METHOD_CONVOLUTION;
  m_iDefaultFilterGeneration = ProcessSignal::FILTER_GENERATION_DIRECT;
#endif
  m_iDefaultZeropad = 1;
  m_iDefaultBackprojector = Backprojector::BPROJ_IDIFF3;
  m_iDefaultInterpolation = Backprojector::INTERP_LINEAR;
  m_iDefaultInterpParam = 1;
  m_iDefaultTrace = Trace::TRACE_NONE;

  m_iDefaultPolarNX = 256;
  m_iDefaultPolarNY = 256;
  m_iDefaultPolarInterpolation = Projections::POLAR_INTERP_BILINEAR;
  m_iDefaultPolarZeropad = 1;
}

ProjectionFileView::~ProjectionFileView(void)
{
}

void
ProjectionFileView::OnProperties (wxCommandEvent& event)
{
  const Projections& rProj = GetDocument()->getProjections();
  std::ostringstream os;
  rProj.printScanInfo(os);
  *theApp->getLog() << os.str().c_str();
  wxMessageDialog dialogMsg (getFrameForChild(), os.str().c_str(), "Projection File Properties", wxOK | wxICON_INFORMATION);
  dialogMsg.ShowModal();
}


void
ProjectionFileView::OnConvertPolar (wxCommandEvent& event)
{
  Projections& rProj = GetDocument()->getProjections();
  DialogGetConvertPolarParameters dialogPolar (getFrameForChild(), "Convert Polar", m_iDefaultPolarNX, m_iDefaultPolarNY,
    m_iDefaultPolarInterpolation, -1);
  if (dialogPolar.ShowModal() == wxID_OK) {
    wxString strInterpolation (dialogPolar.getInterpolationName());
    m_iDefaultPolarNX = dialogPolar.getXSize();
    m_iDefaultPolarNY = dialogPolar.getYSize();
    ImageFileDocument* pPolarDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
    ImageFile& rIF = pPolarDoc->getImageFile();
    if (! pPolarDoc) {
      sys_error (ERR_SEVERE, "Unable to create image file");
      return;
    }
    rIF.setArraySize (m_iDefaultPolarNX, m_iDefaultPolarNY);
    m_iDefaultPolarInterpolation = Projections::convertInterpNameToID (strInterpolation.c_str());
    rProj.convertPolar (rIF, m_iDefaultPolarInterpolation);
    rIF.labelAdd (rProj.getLabel().getLabelString().c_str(), rProj.calcTime());
    std::ostringstream os;
    os << "Convert projection file " << GetFrame()->GetTitle().c_str() << " to polar image: xSize=" 
      << m_iDefaultPolarNX << ", ySize=" << m_iDefaultPolarNY << ", interpolation=" 
      << strInterpolation.c_str();
    *theApp->getLog() << os.str().c_str() << "\n";
    rIF.labelAdd (os.str().c_str());
    if (theApp->getSetModifyNewDocs())
      pPolarDoc->Modify(true);
    pPolarDoc->UpdateAllViews();
    pPolarDoc->GetFirstView()->OnUpdate (this, NULL);
  }
}

void
ProjectionFileView::OnConvertFFTPolar (wxCommandEvent& event)
{
  Projections& rProj = GetDocument()->getProjections();
  DialogGetConvertPolarParameters dialogPolar (getFrameForChild(), "Convert to FFT Polar", m_iDefaultPolarNX, m_iDefaultPolarNY,
    m_iDefaultPolarInterpolation, m_iDefaultPolarZeropad);
  if (dialogPolar.ShowModal() == wxID_OK) {
    wxString strInterpolation (dialogPolar.getInterpolationName());
    m_iDefaultPolarNX = dialogPolar.getXSize();
    m_iDefaultPolarNY = dialogPolar.getYSize();
    m_iDefaultPolarZeropad = dialogPolar.getZeropad();
    ImageFileDocument* pPolarDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
    ImageFile& rIF = pPolarDoc->getImageFile();
    if (! pPolarDoc) {
      sys_error (ERR_SEVERE, "Unable to create image file");
      return;
    }
    rIF.setArraySize (m_iDefaultPolarNX, m_iDefaultPolarNY);
    m_iDefaultPolarInterpolation = Projections::convertInterpNameToID (strInterpolation.c_str());
    rProj.convertFFTPolar (rIF, m_iDefaultPolarInterpolation, m_iDefaultPolarZeropad);
    rIF.labelAdd (rProj.getLabel().getLabelString().c_str(), rProj.calcTime());
    std::ostringstream os;
    os << "Convert projection file " << GetFrame()->GetTitle().c_str() << " to FFT polar image: xSize=" 
      << m_iDefaultPolarNX << ", ySize=" << m_iDefaultPolarNY << ", interpolation=" 
      << strInterpolation.c_str() << ", zeropad=" << m_iDefaultPolarZeropad;
    *theApp->getLog() << os.str().c_str() << "\n";
    rIF.labelAdd (os.str().c_str());
    if (theApp->getSetModifyNewDocs())
      pPolarDoc->Modify(true);
    pPolarDoc->UpdateAllViews();
    pPolarDoc->GetFirstView()->OnUpdate (this, NULL);
  }}

void
ProjectionFileView::OnReconstructFourier (wxCommandEvent& event)
{
  wxMessageBox ("Fourier Reconstruction is not yet supported", "Unimplemented function");
}

void
ProjectionFileView::OnReconstructFBP (wxCommandEvent& event)
{
  DialogGetReconstructionParameters dialogReconstruction (getFrameForChild(), m_iDefaultNX, m_iDefaultNY, m_iDefaultFilter, m_dDefaultFilterParam, m_iDefaultFilterMethod, m_iDefaultFilterGeneration, m_iDefaultZeropad, m_iDefaultInterpolation, m_iDefaultInterpParam, m_iDefaultBackprojector, m_iDefaultTrace);
  
  int retVal = dialogReconstruction.ShowModal();
  if (retVal == wxID_OK) {
    m_iDefaultNX = dialogReconstruction.getXSize();
    m_iDefaultNY = dialogReconstruction.getYSize();
    wxString optFilterName = dialogReconstruction.getFilterName();
    m_iDefaultFilter = SignalFilter::convertFilterNameToID (optFilterName.c_str());
    m_dDefaultFilterParam = dialogReconstruction.getFilterParam();
    wxString optFilterMethodName = dialogReconstruction.getFilterMethodName();
    m_iDefaultFilterMethod = ProcessSignal::convertFilterMethodNameToID(optFilterMethodName.c_str());
    m_iDefaultZeropad = dialogReconstruction.getZeropad();
    wxString optFilterGenerationName = dialogReconstruction.getFilterGenerationName();
    m_iDefaultFilterGeneration = ProcessSignal::convertFilterGenerationNameToID (optFilterGenerationName.c_str());
    wxString optInterpName = dialogReconstruction.getInterpName();
    m_iDefaultInterpolation = Backprojector::convertInterpNameToID (optInterpName.c_str());
    m_iDefaultInterpParam = dialogReconstruction.getInterpParam();
    wxString optBackprojectName = dialogReconstruction.getBackprojectName();
    m_iDefaultBackprojector = Backprojector::convertBackprojectNameToID (optBackprojectName.c_str());
    m_iDefaultTrace = dialogReconstruction.getTrace();
    if (m_iDefaultNX > 0 && m_iDefaultNY > 0) {
      ImageFileDocument* pReconDoc = dynamic_cast<ImageFileDocument*>(theApp->getDocManager()->CreateDocument("untitled.if", wxDOC_SILENT));
      if (! pReconDoc) {
        sys_error (ERR_SEVERE, "Unable to create image file");
        return;
      }
      ImageFile& imageFile = pReconDoc->getImageFile();
      const Projections& rProj = GetDocument()->getProjections();
      imageFile.setArraySize (m_iDefaultNX, m_iDefaultNY);
      
      Reconstructor* pReconstruct = new Reconstructor (rProj, imageFile, optFilterName.c_str(), m_dDefaultFilterParam, optFilterMethodName.c_str(), m_iDefaultZeropad, optFilterGenerationName.c_str(), optInterpName.c_str(), m_iDefaultInterpParam, optBackprojectName.c_str(), m_iDefaultTrace);

      Timer timerRecon;
      if (m_iDefaultTrace > Trace::TRACE_CONSOLE) {
        ReconstructDialog* pDlgReconstruct = new ReconstructDialog (*pReconstruct, rProj, imageFile, m_iDefaultTrace, getFrameForChild());
        for (int iView = 0; iView < rProj.nView(); iView++) {
          ::wxYield();
          ::wxYield();
          if (pDlgReconstruct->isCancelled() || ! pDlgReconstruct->reconstructView (iView)) {
            delete pDlgReconstruct;
            delete pReconstruct;
            pReconDoc->DeleteAllViews();
            return;
          }
          ::wxYield();
          ::wxYield();
          while (pDlgReconstruct->isPaused()) {
            ::wxYield();
            ::wxUsleep(50);
          }
        }
        delete pDlgReconstruct;
      } else {
        wxProgressDialog dlgProgress (wxString("Reconstruction"), wxString("Reconstruction Progress"), rProj.nView() + 1, getFrameForChild(), wxPD_CAN_ABORT);
        for (int i = 0; i < rProj.nView(); i++) {
          pReconstruct->reconstructView (i, 1);
          if (! dlgProgress.Update(i + 1)) {
            delete pReconstruct;
            pReconDoc->DeleteAllViews();
            return;
          }
        }
      }
      delete pReconstruct;
      if (theApp->getSetModifyNewDocs())
        pReconDoc->Modify(true);
      pReconDoc->UpdateAllViews(this);
      ImageFileView* rasterView = dynamic_cast<ImageFileView*>(pReconDoc->GetFirstView());
      if (rasterView) {
        rasterView->getFrame()->SetFocus();
        rasterView->OnUpdate (rasterView, NULL);
      }
      std::ostringstream os;
      os << "Reconstruct " << rProj.getFilename() << ": xSize=" << m_iDefaultNX << ", ySize=" << m_iDefaultNY << ", Filter=" << optFilterName.c_str() << ", FilterParam=" << m_dDefaultFilterParam << ", FilterMethod=" << optFilterMethodName.c_str() << ", FilterGeneration=" << optFilterGenerationName.c_str() << ", Zeropad=" << m_iDefaultZeropad << ", Interpolation=" << optInterpName.c_str() << ", InterpolationParam=" << m_iDefaultInterpParam << ", Backprojection=" << optBackprojectName.c_str();
      *theApp->getLog() << os.str().c_str() << "\n";
      imageFile.labelAdd (rProj.getLabel());
      imageFile.labelAdd (os.str().c_str(), timerRecon.timerEnd());
    }
  }
}


ProjectionFileCanvas* 
ProjectionFileView::CreateCanvas (wxView *view, wxFrame *parent)
{
  ProjectionFileCanvas* pCanvas;
  int width, height;
  parent->GetClientSize(&width, &height);
  
  pCanvas = new ProjectionFileCanvas (dynamic_cast<ProjectionFileView*>(view), parent, wxPoint(0, 0), wxSize(width, height), 0);
  
  pCanvas->SetScrollbars(20, 20, 50, 50);
  pCanvas->SetBackgroundColour(*wxWHITE);
  pCanvas->Clear();
  
  return pCanvas;
}

wxFrame*
ProjectionFileView::CreateChildFrame(wxDocument *doc, wxView *view)
{
#ifdef CTSIM_MDI
  wxDocMDIChildFrame *subframe = new wxDocMDIChildFrame (doc, view, theApp->getMainFrame(), -1, "Projection Frame", wxPoint(10, 10), wxSize(0, 0), wxDEFAULT_FRAME_STYLE);
#else
  wxDocChildFrame *subframe = new wxDocChildFrame (doc, view, theApp->getMainFrame(), -1, "Projection Frame", wxPoint(10, 10), wxSize(0, 0), wxDEFAULT_FRAME_STYLE);
#endif
  theApp->setIconForFrame (subframe);

  wxMenu *file_menu = new wxMenu;
  
  file_menu->Append(MAINMENU_FILE_CREATE_PHANTOM, "Cr&eate Phantom...\tCtrl-P");
  file_menu->Append(MAINMENU_FILE_CREATE_FILTER, "Create &Filter...\tCtrl-F");
  file_menu->Append(wxID_OPEN, "&Open...\tCtrl-O");
  file_menu->Append(wxID_SAVE, "&Save\tCtrl-S");
  file_menu->Append(wxID_SAVEAS, "Save &As...");
  file_menu->Append(wxID_CLOSE, "&Close\tCtrl-W");
  
  file_menu->AppendSeparator();
  file_menu->Append(PJMENU_FILE_PROPERTIES, "P&roperties");
  
  file_menu->AppendSeparator();
  file_menu->Append(wxID_PRINT, "&Print...");
  file_menu->Append(wxID_PRINT_SETUP, "Print &Setup...");
  file_menu->Append(wxID_PREVIEW, "Print Pre&view");
  
  wxMenu *convert_menu = new wxMenu;
  convert_menu->Append (PJMENU_CONVERT_POLAR, "&Polar Image...\tCtrl-L");
  convert_menu->Append (PJMENU_CONVERT_FFT_POLAR, "&FFT->Polar Image...\tCtrl-I");
  
  wxMenu *reconstruct_menu = new wxMenu;
  reconstruct_menu->Append (PJMENU_RECONSTRUCT_FBP, "&Filtered Backprojection...\tCtrl-R");
  reconstruct_menu->Append (PJMENU_RECONSTRUCT_FOURIER, "&Fourier...\tCtrl-E");

  wxMenu *help_menu = new wxMenu;
  help_menu->Append(MAINMENU_HELP_CONTENTS, "&Contents\tF1");
  help_menu->Append(MAINMENU_HELP_TOPICS, "&Topics\tCtrl-H");
  help_menu->Append(MAINMENU_HELP_ABOUT, "&About");
  
  wxMenuBar *menu_bar = new wxMenuBar;
  
  menu_bar->Append (file_menu, "&File");
  menu_bar->Append (convert_menu, "&Convert");
  menu_bar->Append (reconstruct_menu, "&Reconstruct");
  menu_bar->Append (help_menu, "&Help");
  
  subframe->SetMenuBar(menu_bar);  
  subframe->Centre(wxBOTH);

  wxAcceleratorEntry accelEntries[11];
  accelEntries[0].Set (wxACCEL_CTRL, static_cast<int>('O'), wxID_OPEN);
  accelEntries[1].Set (wxACCEL_CTRL, static_cast<int>('S'), wxID_SAVE);
  accelEntries[2].Set (wxACCEL_CTRL, static_cast<int>('W'), wxID_CLOSE);
  accelEntries[3].Set (wxACCEL_CTRL, static_cast<int>('H'), MAINMENU_HELP_TOPICS);
  accelEntries[4].Set (wxACCEL_CTRL, static_cast<int>('P'), MAINMENU_FILE_CREATE_PHANTOM);
  accelEntries[5].Set (wxACCEL_CTRL, static_cast<int>('F'), MAINMENU_FILE_CREATE_FILTER);
  accelEntries[6].Set (wxACCEL_NORMAL, WXK_F1, MAINMENU_HELP_CONTENTS);
  accelEntries[7].Set (wxACCEL_CTRL, static_cast<int>('L'), PJMENU_CONVERT_POLAR);
  accelEntries[8].Set (wxACCEL_CTRL, static_cast<int>('I'), PJMENU_CONVERT_FFT_POLAR);
  accelEntries[9].Set (wxACCEL_CTRL, static_cast<int>('R'), PJMENU_RECONSTRUCT_FBP);
  accelEntries[10].Set (wxACCEL_CTRL, static_cast<int>('E'), PJMENU_RECONSTRUCT_FOURIER);
  wxAcceleratorTable accelTable (11, accelEntries);
  subframe->SetAcceleratorTable (accelTable);
  
  return subframe;
}


bool 
ProjectionFileView::OnCreate(wxDocument *doc, long WXUNUSED(flags) )
{
  m_frame = CreateChildFrame(doc, this);
  SetFrame(m_frame);
  
  int width, height;
  m_frame->GetClientSize(&width, &height);
  m_frame->SetTitle("ProjectionFileView");
  m_canvas = CreateCanvas(this, m_frame);
  
#ifdef __X__
  int x, y;  // X requires a forced resize
  m_frame->GetSize(&x, &y);
  m_frame->SetSize(-1, -1, x, y);
#endif
  
  m_frame->Show(true);
  Activate(true);
  
  return true;
}

void 
ProjectionFileView::OnDraw (wxDC* dc)
{
  wxSize clientSize = m_frame->GetClientSize();
  wxSize bestSize = m_canvas->GetBestSize();

  if (clientSize.x > bestSize.x || clientSize.y > bestSize.y)
    m_frame->SetClientSize (bestSize);

  if (m_bitmap.Ok())
    dc->DrawBitmap (m_bitmap, 0, 0, false);
}


void 
ProjectionFileView::OnUpdate(wxView *WXUNUSED(sender), wxObject *WXUNUSED(hint) )
{
  const Projections& rProj = GetDocument()->getProjections();
  const int nDet = rProj.nDet();
  const int nView = rProj.nView();
  if (nDet != 0 && nView != 0) {
    const DetectorArray& detarray = rProj.getDetectorArray(0);
    const DetectorValue* detval = detarray.detValues();
    double min = detval[0];
    double max = detval[0];
    for (int iy = 0; iy < nView; iy++) {
      const DetectorArray& detarray = rProj.getDetectorArray(iy);
      const DetectorValue* detval = detarray.detValues();
      for (int ix = 0; ix < nDet; ix++) {
        if (min > detval[ix])
          min = detval[ix];
        else if (max < detval[ix])
          max = detval[ix];
      }
    }
    
    unsigned char* imageData = new unsigned char [nDet * nView * 3];
    double scale = (max - min) / 255;
    for (int iy2 = 0; iy2 < nView; iy2++) {
      const DetectorArray& detarray = rProj.getDetectorArray (iy2);
      const DetectorValue* detval = detarray.detValues();
      for (int ix = 0; ix < nDet; ix++) {
        int intensity = static_cast<int>(((detval[ix] - min) / scale) + 0.5);
        intensity = clamp(intensity, 0, 255);
        int baseAddr = (iy2 * nDet + ix) * 3;
        imageData[baseAddr] = imageData[baseAddr+1] = imageData[baseAddr+2] = intensity;
      }
    }
    wxImage image (nDet, nView, imageData, true);
    m_bitmap = image.ConvertToBitmap();
    delete imageData;
    int xSize = nDet;
    int ySize = nView;
    xSize = clamp (xSize, 0, 800);
    ySize = clamp (ySize, 0, 800);
    m_frame->SetClientSize (xSize, ySize);
    m_canvas->SetScrollbars (20, 20, nDet/20, nView/20);
  }
  
  if (m_canvas)
    m_canvas->Refresh();
}

bool 
ProjectionFileView::OnClose (bool deleteWindow)
{
  if (!GetDocument()->Close())
    return false;
  
  m_canvas->Clear();
  // m_canvas->m_pView = NULL;
  m_canvas = NULL;
  wxString s(wxTheApp->GetAppName());
  if (m_frame)
    m_frame->SetTitle(s);
  SetFrame(NULL);
  
  Activate(false);
  
  if (deleteWindow) {
    delete m_frame;
    return true;
  }
  return true;
}



// PlotFileCanvas
PlotFileCanvas::PlotFileCanvas (PlotFileView* v, wxFrame *frame, const wxPoint& pos, const wxSize& size, const long style)
: wxScrolledWindow(frame, -1, pos, size, style)
{
  m_pView = v;
}

void 
PlotFileCanvas::OnDraw(wxDC& dc)
{
  if (m_pView)
    m_pView->OnDraw(& dc);
}


// PlotFileView

IMPLEMENT_DYNAMIC_CLASS(PlotFileView, wxView)

BEGIN_EVENT_TABLE(PlotFileView, wxView)
EVT_MENU(PJMENU_FILE_PROPERTIES, PlotFileView::OnProperties)
EVT_MENU(PLOTMENU_VIEW_SCALE_MINMAX, PlotFileView::OnScaleMinMax)
EVT_MENU(PLOTMENU_VIEW_SCALE_AUTO, PlotFileView::OnScaleAuto)
EVT_MENU(PLOTMENU_VIEW_SCALE_FULL, PlotFileView::OnScaleFull)
END_EVENT_TABLE()

PlotFileView::PlotFileView(void) 
: wxView(), m_canvas(NULL), m_frame(NULL), m_pEZPlot(NULL)
{
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
}

PlotFileView::~PlotFileView(void)
{
  if (m_pEZPlot)
    delete m_pEZPlot;
}

void
PlotFileView::OnProperties (wxCommandEvent& event)
{
  const PlotFile& rPlot = GetDocument()->getPlotFile();
  std::ostringstream os;
  os << "Columns: " << rPlot.getNumColumns() << ", Records: " << rPlot.getNumRecords() << "\n";
  rPlot.printHeadersBrief (os);
  *theApp->getLog() << os.str().c_str();
  wxMessageDialog dialogMsg (getFrameForChild(), os.str().c_str(), "Plot File Properties", wxOK | wxICON_INFORMATION);
  dialogMsg.ShowModal();
}


void 
PlotFileView::OnScaleAuto (wxCommandEvent& event)
{
  const PlotFile& rPlotFile = GetDocument()->getPlotFile();
  double min, max, mean, mode, median, stddev;
  rPlotFile.statistics (1, min, max, mean, mode, median, stddev);
  DialogAutoScaleParameters dialogAutoScale (getFrameForChild(), mean, mode, median, stddev, m_dAutoScaleFactor);
  int iRetVal = dialogAutoScale.ShowModal();
  if (iRetVal == wxID_OK) {
    m_bMinSpecified = true;
    m_bMaxSpecified = true;
    double dMin, dMax;
    if (dialogAutoScale.getMinMax (&dMin, &dMax)) {
      m_dMinPixel = dMin;
      m_dMaxPixel = dMax;
      m_dAutoScaleFactor = dialogAutoScale.getAutoScaleFactor();
      OnUpdate (this, NULL);
    }
  }
}

void 
PlotFileView::OnScaleMinMax (wxCommandEvent& event)
{
  const PlotFile& rPlotFile = GetDocument()->getPlotFile();
  double min;
  double max;

  if (! m_bMinSpecified || ! m_bMaxSpecified) {
    if (! rPlotFile.getMinMax (1, min, max)) {
      *theApp->getLog() << "Error: unable to find Min/Max\n";
      return;
    }
  }
  
  if (m_bMinSpecified)
    min = m_dMinPixel;
  if (m_bMaxSpecified)
    max = m_dMaxPixel;
  
  DialogGetMinMax dialogMinMax (getFrameForChild(), "Set Y-axis Minimum & Maximum", min, max);
  int retVal = dialogMinMax.ShowModal();
  if (retVal == wxID_OK) {
    m_bMinSpecified = true;
    m_bMaxSpecified = true;
    m_dMinPixel = dialogMinMax.getMinimum();
    m_dMaxPixel = dialogMinMax.getMaximum();
    OnUpdate (this, NULL);
  }
}

void 
PlotFileView::OnScaleFull (wxCommandEvent& event)
{
  if (m_bMinSpecified || m_bMaxSpecified) {
    m_bMinSpecified = false;
    m_bMaxSpecified = false;
    OnUpdate (this, NULL);
  }
}


PlotFileCanvas* 
PlotFileView::CreateCanvas (wxView *view, wxFrame *parent)
{
  PlotFileCanvas* pCanvas;
  int width, height;
  parent->GetClientSize(&width, &height);
  
  pCanvas = new PlotFileCanvas (dynamic_cast<PlotFileView*>(view), parent, wxPoint(0, 0), wxSize(width, height), 0);
  
  pCanvas->SetBackgroundColour(*wxWHITE);
  pCanvas->Clear();
  
  return pCanvas;
}

wxFrame*
PlotFileView::CreateChildFrame(wxDocument *doc, wxView *view)
{
#ifdef CTSIM_MDI
  wxDocMDIChildFrame *subframe = new wxDocMDIChildFrame (doc, view, theApp->getMainFrame(), -1, "Plot Frame", wxPoint(10, 10), wxSize(500, 300), wxDEFAULT_FRAME_STYLE);
#else
  wxDocChildFrame *subframe = new wxDocChildFrame(doc, view, theApp->getMainFrame(), -1, "Plot Frame", wxPoint(10, 10), wxSize(500, 300), wxDEFAULT_FRAME_STYLE);
#endif
  theApp->setIconForFrame (subframe);

  wxMenu *file_menu = new wxMenu;
  
  file_menu->Append(MAINMENU_FILE_CREATE_PHANTOM, "Cr&eate Phantom...\tCtrl-P");
  file_menu->Append(MAINMENU_FILE_CREATE_FILTER, "Create &Filter...\tCtrl-F");
  file_menu->Append(wxID_OPEN, "&Open...\tCtrl-O");
  file_menu->Append(wxID_SAVE, "&Save\tCtrl-S");
  file_menu->Append(wxID_SAVEAS, "Save &As...");
  file_menu->Append(wxID_CLOSE, "&Close\tCtrl-W");
  
  file_menu->AppendSeparator();
  file_menu->Append(PJMENU_FILE_PROPERTIES, "P&roperties");
  
  file_menu->AppendSeparator();
  file_menu->Append(wxID_PRINT, "&Print...");
  file_menu->Append(wxID_PRINT_SETUP, "Print &Setup...");
  file_menu->Append(wxID_PREVIEW, "Print Pre&view");
  
  wxMenu *view_menu = new wxMenu;
  view_menu->Append(PLOTMENU_VIEW_SCALE_MINMAX, "Display Scale &Set...\tCtrl-E");
  view_menu->Append(PLOTMENU_VIEW_SCALE_AUTO, "Display Scale &Auto...\tCtrl-A");
  view_menu->Append(PLOTMENU_VIEW_SCALE_FULL, "Display &Full Scale\tCtrl-U");
  
  wxMenu *help_menu = new wxMenu;
  help_menu->Append(MAINMENU_HELP_CONTENTS, "&Contents\tF1");
  help_menu->Append(MAINMENU_HELP_TOPICS, "&Topics\tCtrl-H");
  help_menu->Append(MAINMENU_HELP_ABOUT, "&About");
  
  wxMenuBar *menu_bar = new wxMenuBar;
  
  menu_bar->Append(file_menu, "&File");
  menu_bar->Append(view_menu, "&View");
  menu_bar->Append(help_menu, "&Help");
  
  subframe->SetMenuBar(menu_bar);
  subframe->Centre(wxBOTH);

  wxAcceleratorEntry accelEntries[10];
  accelEntries[0].Set (wxACCEL_CTRL, static_cast<int>('O'), wxID_OPEN);
  accelEntries[1].Set (wxACCEL_CTRL, static_cast<int>('S'), wxID_SAVE);
  accelEntries[2].Set (wxACCEL_CTRL, static_cast<int>('W'), wxID_CLOSE);
  accelEntries[3].Set (wxACCEL_CTRL, static_cast<int>('H'), MAINMENU_HELP_TOPICS);
  accelEntries[4].Set (wxACCEL_CTRL, static_cast<int>('P'), MAINMENU_FILE_CREATE_PHANTOM);
  accelEntries[5].Set (wxACCEL_CTRL, static_cast<int>('F'), MAINMENU_FILE_CREATE_FILTER);
  accelEntries[6].Set (wxACCEL_NORMAL, WXK_F1, MAINMENU_HELP_CONTENTS);
  accelEntries[7].Set (wxACCEL_CTRL, static_cast<int>('E'), PLOTMENU_VIEW_SCALE_MINMAX);
  accelEntries[8].Set (wxACCEL_CTRL, static_cast<int>('A'), PLOTMENU_VIEW_SCALE_AUTO);
  accelEntries[9].Set (wxACCEL_CTRL, static_cast<int>('U'), PLOTMENU_VIEW_SCALE_FULL);
  wxAcceleratorTable accelTable (10, accelEntries);
  subframe->SetAcceleratorTable (accelTable);
  
  return subframe;
}


bool 
PlotFileView::OnCreate (wxDocument *doc, long WXUNUSED(flags) )
{
  m_frame = CreateChildFrame(doc, this);
  SetFrame(m_frame);
  
  m_bMinSpecified = false;
  m_bMaxSpecified = false;
  m_dAutoScaleFactor = 1.;
  
  int width, height;
  m_frame->GetClientSize(&width, &height);
  m_frame->SetTitle ("Plot File");
  m_canvas = CreateCanvas (this, m_frame);
  
#ifdef __X__
  int x, y;  // X requires a forced resize
  m_frame->GetSize(&x, &y);
  m_frame->SetSize(-1, -1, x, y);
#endif
  
  m_frame->Show(true);
  Activate(true);
   
  return true;
}

void 
PlotFileView::OnDraw (wxDC* dc)
{
  const PlotFile& rPlotFile = GetDocument()->getPlotFile();
  const int iNColumns = rPlotFile.getNumColumns();
  const int iNRecords = rPlotFile.getNumRecords();
  
  if (iNColumns > 0 && iNRecords > 0) {
    int xsize, ysize;
    m_canvas->GetClientSize (&xsize, &ysize);
    SGPDriver driver (dc, xsize, ysize);
    SGP sgp (driver);
    if (m_pEZPlot)
      m_pEZPlot->plot (&sgp);
  }
}


void 
PlotFileView::OnUpdate (wxView *WXUNUSED(sender), wxObject *WXUNUSED(hint) )
{
    const PlotFile& rPlotFile = GetDocument()->getPlotFile();
    const int iNColumns = rPlotFile.getNumColumns();
    const int iNRecords = rPlotFile.getNumRecords();
    
    if (iNColumns > 0 && iNRecords > 0) {
      if (m_pEZPlot)
        delete m_pEZPlot;
      m_pEZPlot = new EZPlot;
      
      for (unsigned int iEzset = 0; iEzset < rPlotFile.getNumEzsetCommands(); iEzset++)
        m_pEZPlot->ezset (rPlotFile.getEzsetCommand (iEzset));
      
      if (m_bMinSpecified) {
        std::ostringstream os;
        os << "ymin " << m_dMinPixel;
        m_pEZPlot->ezset (os.str());
      }
      
      if (m_bMaxSpecified) {
        std::ostringstream os;
        os << "ymax " << m_dMaxPixel;
        m_pEZPlot->ezset (os.str());
      }
      
      m_pEZPlot->ezset("box");
      m_pEZPlot->ezset("grid");
      
      double* pdXaxis = new double [iNRecords];
      rPlotFile.getColumn (0, pdXaxis);
      
      double* pdY = new double [iNRecords];
      for (int iCol = 1; iCol < iNColumns; iCol++) {
        rPlotFile.getColumn (iCol, pdY);
        m_pEZPlot->addCurve (pdXaxis, pdY, iNRecords);
      }
      
      delete pdXaxis;
      delete pdY;
    }

    if (m_canvas)
      m_canvas->Refresh();
}

bool 
PlotFileView::OnClose (bool deleteWindow)
{
  if (!GetDocument()->Close())
    return false;
  
  m_canvas->Clear();
  // m_canvas->m_pView = NULL;
  m_canvas = NULL;
  wxString s(wxTheApp->GetAppName());
  if (m_frame)
    m_frame->SetTitle(s);
  SetFrame(NULL);
  
  Activate(false);
  
  if (deleteWindow) {
    delete m_frame;
    return true;
  }
  return true;
}

